/*************************************************************************\
*                  Copyright (C) Michael Kerrisk, 2014.                   *
*                                                                         *
* This program is free software. You may use, modify, and redistribute it *
* under the terms of the GNU Affero General Public License as published   *
* by the Free Software Foundation, either version 3 or (at your option)   *
* any later version. This program is distributed without any warranty.    *
* See the file COPYING.agpl-v3 for details.                               *
\*************************************************************************/

/* Listing 17-1 */

  1 #include <acl/libacl.h>
  2 #include <sys/acl.h>
  3 #include "ugid_functions.h"
  4 #include "tlpi_hdr.h"
  5
  6 static void
  7 usageError(char *progName)
  8 {
  9     fprintf(stderr, "Usage: %s [-d] filename\n", progName);
 10     exit(EXIT_FAILURE);
 11 }
 12
 13 int
 14 main(int argc, char *argv[])
 15 {
 16     acl_t acl;
 17     acl_type_t type;
 18     acl_entry_t entry;
 19     acl_tag_t tag;
 20     uid_t *uidp;
 21     gid_t *gidp;
 22     acl_permset_t permset;
 23     char *name;
 24     int entryId, permVal, opt;
 25
 26     type = ACL_TYPE_ACCESS;
 27     while ((opt = getopt(argc, argv, "d")) != -1) {
 28         switch (opt) {
 29         case 'd': type = ACL_TYPE_DEFAULT;      break;
 30         case '?': usageError(argv[0]);
 31         }
 32     }
 33
 34     if (optind + 1 != argc)
 35         usageError(argv[0]);
 36
 37     acl = acl_get_file(argv[optind], type);
 38     if (acl == NULL)
 39         errExit("acl_get_file");
 40
 41     /* Walk through each entry in this ACL */
 42
 43     for (entryId = ACL_FIRST_ENTRY; ; entryId = ACL_NEXT_ENTRY) {
 44
 45         if (acl_get_entry(acl, entryId, &entry) != 1)
 46             break;                      /* Exit on error or no more entries */
 47
 48         /* Retrieve and display tag type */
 49
 50         if (acl_get_tag_type(entry, &tag) == -1)
 51             errExit("acl_get_tag_type");
 52
 53         printf("%-12s", (tag == ACL_USER_OBJ) ?  "user_obj" :
 54                         (tag == ACL_USER) ?      "user" :
 55                         (tag == ACL_GROUP_OBJ) ? "group_obj" :
 56                         (tag == ACL_GROUP) ?     "group" :
 57                         (tag == ACL_MASK) ?      "mask" :
 58                         (tag == ACL_OTHER) ?     "other" : "???");
 59
 60         /* Retrieve and display optional tag qualifier */
 61
 62         if (tag == ACL_USER) {
 63             uidp = acl_get_qualifier(entry);
 64             if (uidp == NULL)
 65                 errExit("acl_get_qualifier");
 66
 67             name = userNameFromId(*uidp);
 68             if (name == NULL)
 69                 printf("%-8d ", *uidp);
 70             else
 71                 printf("%-8s ", name);
 72
 73             if (acl_free(uidp) == -1)
 74                 errExit("acl_free");
 75
 76         } else if (tag == ACL_GROUP) {
 77             gidp = acl_get_qualifier(entry);
 78             if (gidp == NULL)
 79                 errExit("acl_get_qualifier");
 80
 81             name = groupNameFromId(*gidp);
 82             if (name == NULL)
 83                 printf("%-8d ", *gidp);
 84             else
 85                 printf("%-8s ", name);
 86
 87             if (acl_free(gidp) == -1)
 88                 errExit("acl_free");
 89
 90         } else {
 91             printf("         ");
 92         }
 93
 94         /* Retrieve and display permissions */
 95
 96         if (acl_get_permset(entry, &permset) == -1)
 97             errExit("acl_get_permset");
 98
 99         permVal = acl_get_perm(permset, ACL_READ);
100         if (permVal == -1)
101             errExit("acl_get_perm - ACL_READ");
102         printf("%c", (permVal == 1) ? 'r' : '-');
103         permVal = acl_get_perm(permset, ACL_WRITE);
104         if (permVal == -1)
105             errExit("acl_get_perm - ACL_WRITE");
106         printf("%c", (permVal == 1) ? 'w' : '-');
107         permVal = acl_get_perm(permset, ACL_EXECUTE);
108         if (permVal == -1)
109             errExit("acl_get_perm - ACL_EXECUTE");
110         printf("%c", (permVal == 1) ? 'x' : '-');
111
112         printf("\n");
113     }
114
115     if (acl_free(acl) == -1)
116         errExit("acl_free");
117
118     exit(EXIT_SUCCESS);
119 }
