/*************************************************************************\
*                  Copyright (C) Michael Kerrisk, 2014.                   *
*                                                                         *
* This program is free software. You may use, modify, and redistribute it *
* under the terms of the GNU Affero General Public License as published   *
* by the Free Software Foundation, either version 3 or (at your option)   *
* any later version. This program is distributed without any warranty.    *
* See the file COPYING.agpl-v3 for details.                               *
\*************************************************************************/

/* Solution for Exercise 63-3 */

  1 #include <sys/time.h>
  2 #include <sys/select.h>
  3 #include <sys/msg.h>
  4 #include <signal.h>
  5 #include <stddef.h>
  6 #include "tlpi_hdr.h"
  7
  8 #define BUF_SIZE 200
  9
 10 /* Total size of the 'pbuf' struct must not exceed PIPE_BUF, otherwise
 11    writes by multiple children may not be atomic, with the result that
 12    messages are broken up and interleaved. */
 13
 14 #define MAX_MTEXT 400
 15
 16 struct pbuf {
 17     int msqid;                  /* Origin of message */
 18     int len;                    /* Number of bytes used in mtext */
 19     long mtype;                 /* Message type */
 20     char mtext[MAX_MTEXT];      /* Message body */
 21 };
 22
 23 /* Function called by child: monitors message queue identified by
 24    'msqid', copying every message to the pipe identified by 'fd'. */
 25
 26 static void
 27 childMon(int msqid, int fd)
 28 {
 29     struct pbuf pmsg;
 30     ssize_t msgLen;
 31     size_t wlen;
 32
 33     for (;;) {
 34         msgLen = msgrcv(msqid, &pmsg.mtype, MAX_MTEXT, 0, 0);
 35         if (msgLen == -1)
 36             errExit("msgrcv");
 37
 38         /* We add some info to the message read by msgrcv() before
 39            writing to the pipe. */
 40
 41         pmsg.msqid = msqid;
 42         pmsg.len = msgLen;      /* So parent knows how much to read from pipe */
 43
 44         wlen = offsetof(struct pbuf, mtext) + msgLen;
 45         /* Or: wlen = &pmsg.mtext - &pmsg + msgLen */
 46
 47         if (write(fd, &pmsg, wlen) != wlen)
 48             fatal("partial/failed write to pipe");
 49     }
 50 }
 51
 52 int
 53 main(int argc, char *argv[])
 54 {
 55     fd_set readfds;
 56     int ready, nfds, j;
 57     int pfd[2];                 /* Pipe used to transfer messages from
 58                                    children to parent */
 59     ssize_t numRead;
 60     char buf[BUF_SIZE];
 61     struct pbuf pmsg;
 62
 63     if (argc < 2 || strcmp(argv[1], "--help") == 0)
 64         usageErr("%s msqid...\n", argv[0]);
 65
 66     if (pipe(pfd) == -1)
 67         errExit("pipe");
 68
 69     /* Create one child for each message queue being monitored */
 70
 71     for (j = 1; j < argc; j++) {
 72         switch (fork()) {
 73         case -1:
 74             errMsg("fork");
 75             killpg(0, SIGTERM);
 76             _exit(EXIT_FAILURE);        /* NOTREACHED */
 77
 78         case 0:
 79             childMon(getInt(argv[j], 0, "msqid"), pfd[1]);
 80             _exit(EXIT_FAILURE);        /* NOTREACHED */
 81
 82         default:
 83             break;
 84         }
 85     }
 86
 87     /* Parent falls through to here */
 88
 89     for (;;) {
 90         FD_ZERO(&readfds);
 91         FD_SET(STDIN_FILENO, &readfds);
 92         FD_SET(pfd[0], &readfds);
 93         nfds = max(STDIN_FILENO, pfd[0]) + 1;
 94
 95         ready = select(nfds, &readfds, NULL, NULL, NULL);
 96         if (ready == -1)
 97             errExit("select");
 98
 99         /* Check if terminal fd is ready */
100
101         if (FD_ISSET(STDIN_FILENO, &readfds)) {
102             numRead = read(STDIN_FILENO, buf, BUF_SIZE - 1);
103             if (numRead == -1)
104                 errExit("read stdin");
105
106             buf[numRead] = '\0';
107             printf("Read from terminal: %s", buf);
108             if (numRead > 0 && buf[numRead - 1] != '\n')
109                 printf("\n");
110         }
111
112         /* Check if pipe fd is ready */
113
114         if (FD_ISSET(pfd[0], &readfds)) {
115             numRead = read(pfd[0], &pmsg, offsetof(struct pbuf, mtext));
116             if (numRead == -1)
117                 errExit("read pipe");
118             if (numRead == 0)
119                 fatal("EOF on pipe");
120
121             numRead = read(pfd[0], &pmsg.mtext, pmsg.len);
122             if (numRead == -1)
123                 errExit("read pipe");
124             if (numRead == 0)
125                 fatal("EOF on pipe");
126
127             printf("MQ %d: type=%ld length=%d <%.*s>\n", pmsg.msqid,
128                     pmsg.mtype, pmsg.len, pmsg.len, pmsg.mtext);
129         }
130     }
131 }
