/*************************************************************************\
*                  Copyright (C) Michael Kerrisk, 2014.                   *
*                                                                         *
* This program is free software. You may use, modify, and redistribute it *
* under the terms of the GNU Affero General Public License as published   *
* by the Free Software Foundation, either version 3 or (at your option)   *
* any later version. This program is distributed without any warranty.    *
* See the file COPYING.agpl-v3 for details.                               *
\*************************************************************************/

/* Listing 39-1 */

  1 #define _BSD_SOURCE             /* Get getpass() declaration from <unistd.h> */
  2 #define _XOPEN_SOURCE           /* Get crypt() declaration from <unistd.h> */
  3 #include <sys/capability.h>
  4 #include <unistd.h>
  5 #include <limits.h>
  6 #include <pwd.h>
  7 #include <shadow.h>
  8 #include "tlpi_hdr.h"
  9
 10 /* Change setting of capability in caller's effective capabilities */
 11
 12 static int
 13 modifyCap(int capability, int setting)
 14 {
 15     cap_t caps;
 16     cap_value_t capList[1];
 17
 18     /* Retrieve caller's current capabilities */
 19
 20     caps = cap_get_proc();
 21     if (caps == NULL)
 22         return -1;
 23
 24     /* Change setting of 'capability' in the effective set of 'caps'. The
 25        third argument, 1, is the number of items in the array 'capList'. */
 26
 27     capList[0] = capability;
 28     if (cap_set_flag(caps, CAP_EFFECTIVE, 1, capList, setting) == -1) {
 29         cap_free(caps);
 30         return -1;
 31     }
 32
 33     /* Push modified capability sets back to kernel, to change
 34        caller's capabilities */
 35
 36     if (cap_set_proc(caps) == -1) {
 37         cap_free(caps);
 38         return -1;
 39     }
 40
 41     /* Free the structure that was allocated by libcap */
 42
 43     if (cap_free(caps) == -1)
 44         return -1;
 45
 46     return 0;
 47 }
 48
 49 static int              /* Raise capability in caller's effective set */
 50 raiseCap(int capability)
 51 {
 52     return modifyCap(capability, CAP_SET);
 53 }
 54
 55 /* An analogous dropCap() (unneeded in this program), could be
 56    defined as: modifyCap(capability, CAP_CLEAR); */
 57
 58 static int              /* Drop all capabilities from all sets */
 59 dropAllCaps(void)
 60 {
 61     cap_t empty;
 62     int s;
 63
 64     empty = cap_init();
 65     if (empty == NULL)
 66         return -1;
 67
 68     s = cap_set_proc(empty);
 69
 70     if (cap_free(empty) == -1)
 71         return -1;
 72
 73     return s;
 74 }
 75
 76 int
 77 main(int argc, char *argv[])
 78 {
 79     char *username, *password, *encrypted, *p;
 80     struct passwd *pwd;
 81     struct spwd *spwd;
 82     Boolean authOk;
 83     size_t len;
 84     long lnmax;
 85
 86     lnmax = sysconf(_SC_LOGIN_NAME_MAX);
 87     if (lnmax == -1)                        /* If limit is indeterminate */
 88         lnmax = 256;                        /* make a guess */
 89
 90     username = malloc(lnmax);
 91     if (username == NULL)
 92         errExit("malloc");
 93
 94     printf("Username: ");
 95     fflush(stdout);
 96     if (fgets(username, lnmax, stdin) == NULL)
 97         exit(EXIT_FAILURE);                 /* Exit on EOF */
 98
 99     len = strlen(username);
100     if (username[len - 1] == '\n')
101         username[len - 1] = '\0';           /* Remove trailing '\n' */
102
103     pwd = getpwnam(username);
104     if (pwd == NULL)
105         fatal("couldn't get password record");
106
107     /* Only raise CAP_DAC_READ_SEARCH for as long as we need it */
108
109     if (raiseCap(CAP_DAC_READ_SEARCH) == -1)
110         fatal("raiseCap() failed");
111
112     spwd = getspnam(username);
113     if (spwd == NULL && errno == EACCES)
114         fatal("no permission to read shadow password file");
115
116     /* At this point, we won't need any more capabilities,
117        so drop all capabilities from all sets */
118
119     if (dropAllCaps() == -1)
120         fatal("dropAllCaps() failed");
121
122     if (spwd != NULL)           /* If there is a shadow password record */
123         pwd->pw_passwd = spwd->sp_pwdp;     /* Use the shadow password */
124
125     password = getpass("Password: ");
126
127     /* Encrypt password and erase cleartext version immediately */
128
129     encrypted = crypt(password, pwd->pw_passwd);
130     for (p = password; *p != '\0'; )
131         *p++ = '\0';
132
133     if (encrypted == NULL)
134         errExit("crypt");
135
136     authOk = strcmp(encrypted, pwd->pw_passwd) == 0;
137     if (!authOk) {
138         printf("Incorrect password\n");
139         exit(EXIT_FAILURE);
140     }
141
142     printf("Successfully authenticated: UID=%ld\n", (long) pwd->pw_uid);
143
144     /* Now do authenticated work... */
145
146     exit(EXIT_SUCCESS);
147 }
