/*************************************************************************\
*                  Copyright (C) Michael Kerrisk, 2014.                   *
*                                                                         *
* This program is free software. You may use, modify, and redistribute it *
* under the terms of the GNU Affero General Public License as published   *
* by the Free Software Foundation, either version 3 or (at your option)   *
* any later version. This program is distributed without any warranty.    *
* See the file COPYING.agpl-v3 for details.                               *
\*************************************************************************/

/* Solution for Exercise 37-1 */

 1 #include <syslog.h>
 2 #include "tlpi_hdr.h"
 3
 4 static void
 5 usageError(const char *progName)
 6 {
 7     fprintf(stderr, "Usage: %s [-p] [-e] [-l level] \"message\"\n", progName);
 8     fprintf(stderr, "    -p   log PID\n");
 9     fprintf(stderr, "    -e   log to stderr also\n");
10     fprintf(stderr, "    -l   level (g=EMERG; a=ALERT; c=CRIT; e=ERR\n");
11     fprintf(stderr, "                w=WARNING; n=NOTICE; i=INFO; d=DEBUG)\n");
12     exit(EXIT_FAILURE);
13 }
14
15 int
16 main(int argc, char *argv[])
17 {
18     int level, options, opt;
19
20     options = 0;
21     level = LOG_INFO;
22
23     while ((opt = getopt(argc, argv, "l:pe")) != -1) {
24         switch (opt) {
25         case 'l':
26             switch (optarg[0]) {
27             case 'a': level = LOG_ALERT;        break;
28             case 'c': level = LOG_CRIT;         break;
29             case 'e': level = LOG_ERR;          break;
30             case 'w': level = LOG_WARNING;      break;
31             case 'n': level = LOG_NOTICE;       break;
32             case 'i': level = LOG_INFO;         break;
33             case 'd': level = LOG_DEBUG;        break;
34             default:  cmdLineErr("Bad facility: %c\n", optarg[0]);
35             }
36             break;
37
38         case 'p':
39             options |= LOG_PID;
40             break;
41
42         case 'e':
43             options |= LOG_PERROR;
44             break;
45
46         default:
47             fprintf(stderr, "Bad option\n");
48             usageError(argv[0]);
49         }
50     }
51
52     if (argc != optind + 1)
53         usageError(argv[0]);
54
55     openlog(argv[0], options, LOG_USER);
56     syslog(LOG_USER | level, "%s", argv[optind]);
57     closelog();
58
59     exit(EXIT_SUCCESS);
60 }
