/*************************************************************************\
*                  Copyright (C) Michael Kerrisk, 2014.                   *
*                                                                         *
* This program is free software. You may use, modify, and redistribute it *
* under the terms of the GNU Affero General Public License as published   *
* by the Free Software Foundation, either version 3 or (at your option)   *
* any later version. This program is distributed without any warranty.    *
* See the file COPYING.agpl-v3 for details.                               *
\*************************************************************************/

/* Solution for Exercise 18-4 */

 1 #include <limits.h>
 2 #include <dirent.h>
 3 #include <stddef.h>
 4 #include "tlpi_hdr.h"
 5
 6 static void             /* List all files in directory 'dirpath' */
 7 listFiles(const char *dirpath)
 8 {
 9     DIR *dirp;
10     Boolean isCurrent;          /* True if 'dirpath' is "." */
11     struct dirent *result, *entryp;
12     int nameMax;
13
14     isCurrent = strcmp(dirpath, ".") == 0;
15
16     /* On Linux, NAME_MAX is defined in <limits.h>. However, this limit
17        may vary across file systems, so we really should use pathconf()
18        to find the true limit for this file system. */
19
20     nameMax = pathconf(dirpath, _PC_NAME_MAX);
21     if (nameMax == -1)          /* Indeterminate or error */
22         nameMax = 255;          /* So take a guess */
23
24     entryp = malloc(offsetof(struct dirent, d_name) + nameMax + 1);
25     if (entryp == NULL)
26         errExit("malloc");
27
28     /* Open the directory - on failure print an error and return */
29
30     dirp = opendir(dirpath);
31     if (dirp == NULL) {
32         errMsg("opendir failed on '%s'", dirpath);
33         return;
34     }
35
36     /* Look at each of the entries in this directory */
37
38     for (;;) {
39         errno = readdir_r(dirp, entryp, &result);
40         if (errno != 0)
41             errExit("readdir_r");
42
43         if (result == NULL)     /* End of stream */
44             break;
45
46         /* Skip . and .. */
47
48         if (strcmp(entryp->d_name, ".") == 0 ||
49                 strcmp(entryp->d_name, "..") == 0)
50             continue;
51
52         /* Print directory + filename */
53
54         if (!isCurrent) printf("%s/", dirpath);
55         printf("%s\n", entryp->d_name);
56     }
57
58     if (closedir(dirp) == -1)
59         errMsg("closedir");
60 }
61
62 int
63 main(int argc, char *argv[])
64 {
65     if (argc == 1)              /* No arguments - use current directory */
66         listFiles(".");
67     else
68         for (argv++; *argv; argv++)
69             listFiles(*argv);
70     exit(EXIT_SUCCESS);
71 }
