/*************************************************************************\
*                  Copyright (C) Michael Kerrisk, 2014.                   *
*                                                                         *
* This program is free software. You may use, modify, and redistribute it *
* under the terms of the GNU Affero General Public License as published   *
* by the Free Software Foundation, either version 3 or (at your option)   *
* any later version. This program is distributed without any warranty.    *
* See the file COPYING.agpl-v3 for details.                               *
\*************************************************************************/

/* Listing 13-1 */

 1 #define _GNU_SOURCE     /* Obtain O_DIRECT definition from <fcntl.h> */
 2 #include <fcntl.h>
 3 #include <malloc.h>
 4 #include "tlpi_hdr.h"
 5
 6 int
 7 main(int argc, char *argv[])
 8 {
 9     int fd;
10     ssize_t numRead;
11     size_t length, alignment;
12     off_t offset;
13     char *buf;
14
15     if (argc < 3 || strcmp(argv[1], "--help") == 0)
16         usageErr("%s file length [offset [alignment]]\n", argv[0]);
17
18     length = getLong(argv[2], GN_ANY_BASE, "length");
19     offset = (argc > 3) ? getLong(argv[3], GN_ANY_BASE, "offset") : 0;
20     alignment = (argc > 4) ? getLong(argv[4], GN_ANY_BASE, "alignment") : 4096;
21
22     fd = open(argv[1], O_RDONLY | O_DIRECT);
23     if (fd == -1)
24         errExit("open");
25
26     /* memalign() allocates a block of memory aligned on an address that
27        is a multiple of its first argument. By specifying this argument as
28        2 * 'alignment' and then adding 'alignment' to the returned pointer,
29        we ensure that 'buf' is aligned on a non-power-of-two multiple of
30        'alignment'. We do this to ensure that if, for example, we ask
31        for a 256-byte aligned buffer, we don't accidentally get
32        a buffer that is also aligned on a 512-byte boundary. */
33
34     buf = memalign(alignment * 2, length + alignment);
35     if (buf == NULL)
36         errExit("memalign");
37
38     buf += alignment;
39
40     if (lseek(fd, offset, SEEK_SET) == -1)
41         errExit("lseek");
42
43     numRead = read(fd, buf, length);
44     if (numRead == -1)
45         errExit("read");
46     printf("Read %ld bytes\n", (long) numRead);
47
48     exit(EXIT_SUCCESS);
49 }
