 1 /*************************************************************************\
 2 *                  Copyright (C) Michael Kerrisk, 2014.                   *
 3 *                                                                         *
 4 * This program is free software. You may use, modify, and redistribute it *
 5 * under the terms of the GNU Lesser General Public License as published   *
 6 * by the Free Software Foundation, either version 3 or (at your option)   *
 7 * any later version. This program is distributed without any warranty.    *
 8 * See the files COPYING.lgpl-v3 and COPYING.gpl-v3 for details.           *
 9 \*************************************************************************/
10
11 /* Listing 55-3 */
12
13  1 #include <fcntl.h>
14  2 #include "region_locking.h"             /* Declares functions defined here */
15  3
16  4 /* Lock a file region (private; public interfaces below) */
17  5
18  6 static int
19  7 lockReg(int fd, int cmd, int type, int whence, int start, off_t len)
20  8 {
21  9     struct flock fl;
22 10
23 11     fl.l_type = type;
24 12     fl.l_whence = whence;
25 13     fl.l_start = start;
26 14     fl.l_len = len;
27 15
28 16     return fcntl(fd, cmd, &fl);
29 17 }
30 18
31 19 int                     /* Lock a file region using nonblocking F_SETLK */
32 20 lockRegion(int fd, int type, int whence, int start, int len)
33 21 {
34 22     return lockReg(fd, F_SETLK, type, whence, start, len);
35 23 }
36 24
37 25 int                     /* Lock a file region using blocking F_SETLKW */
38 26 lockRegionWait(int fd, int type, int whence, int start, int len)
39 27 {
40 28     return lockReg(fd, F_SETLKW, type, whence, start, len);
41 29 }
42 30
43 31 /* Test if a file region is lockable. Return 0 if lockable, or
44 32    PID of process holding incompatible lock, or -1 on error. */
45 33
46 34 pid_t
47 35 regionIsLocked(int fd, int type, int whence, int start, int len)
48 36 {
49 37     struct flock fl;
50 38
51 39     fl.l_type = type;
52 40     fl.l_whence = whence;
53 41     fl.l_start = start;
54 42     fl.l_len = len;
55 43
56 44     if (fcntl(fd, F_GETLK, &fl) == -1)
57 45         return -1;
58 46
59 47     return (fl.l_type == F_UNLCK) ? 0 : fl.l_pid;
60 48 }
