/*************************************************************************\
*                  Copyright (C) Michael Kerrisk, 2014.                   *
*                                                                         *
* This program is free software. You may use, modify, and redistribute it *
* under the terms of the GNU Affero General Public License as published   *
* by the Free Software Foundation, either version 3 or (at your option)   *
* any later version. This program is distributed without any warranty.    *
* See the file COPYING.agpl-v3 for details.                               *
\*************************************************************************/

/* Listing 55-1 */

 1 #include <sys/file.h>
 2 #include <fcntl.h>
 3 #include "curr_time.h"                  /* Declaration of currTime() */
 4 #include "tlpi_hdr.h"
 5
 6 int
 7 main(int argc, char *argv[])
 8 {
 9     int fd, lock;
10     const char *lname;
11
12     if (argc < 3 || strcmp(argv[1], "--help") == 0 ||
13             strchr("sx", argv[2][0]) == NULL)
14         usageErr("%s file lock [sleep-time]\n"
15                  "    'lock' is 's' (shared) or 'x' (exclusive)\n"
16                  "        optionally followed by 'n' (nonblocking)\n"
17                  "    'sleep-time' specifies time to hold lock\n", argv[0]);
18
19     lock = (argv[2][0] == 's') ? LOCK_SH : LOCK_EX;
20     if (argv[2][1] == 'n')
21         lock |= LOCK_NB;
22
23     fd = open(argv[1], O_RDONLY);               /* Open file to be locked */
24     if (fd == -1)
25         errExit("open");
26
27     lname = (lock & LOCK_SH) ? "LOCK_SH" : "LOCK_EX";
28
29     printf("PID %ld: requesting %s at %s\n", (long) getpid(), lname,
30             currTime("%T"));
31
32     if (flock(fd, lock) == -1) {
33         if (errno == EWOULDBLOCK)
34             fatal("PID %ld: already locked - bye!", (long) getpid());
35         else
36             errExit("flock (PID=%ld)", (long) getpid());
37     }
38
39     printf("PID %ld: granted    %s at %s\n", (long) getpid(), lname,
40             currTime("%T"));
41
42     sleep((argc > 3) ? getInt(argv[3], GN_NONNEG, "sleep-time") : 10);
43
44     printf("PID %ld: releasing  %s at %s\n", (long) getpid(), lname,
45             currTime("%T"));
46     if (flock(fd, LOCK_UN) == -1)
47         errExit("flock");
48
49     exit(EXIT_SUCCESS);
50 }
