/*************************************************************************\
*                  Copyright (C) Michael Kerrisk, 2014.                   *
*                                                                         *
* This program is free software. You may use, modify, and redistribute it *
* under the terms of the GNU Affero General Public License as published   *
* by the Free Software Foundation, either version 3 or (at your option)   *
* any later version. This program is distributed without any warranty.    *
* See the file COPYING.agpl-v3 for details.                               *
\*************************************************************************/

/* Solution for Exercise 15-7 */

 1 #define _GNU_SOURCE
 2 #include <fcntl.h>
 3 #include <sys/ioctl.h>
 4 #include <linux/fs.h>
 5 #include "tlpi_hdr.h"
 6
 7 static void
 8 usageError(const char *progName)
 9 {
10     fprintf(stderr, "Usage: %s {+-=}{attrib-chars} file...\n\n", progName);
11 #define fpe(str) fprintf(stderr, "    " str)            /* Shorter! */
12     fpe("+ add attribute; - remove attribute; "
13                         "= set attributes absolutely\n\n");
14     fpe("'attrib-chars' contains one or more of:\n");
15     fpe("    a   Force open() to include O_APPEND "
16                         "(privilege required)\n");
17     fpe("    A   Do not update last access time\n");
18     fpe("    c   Compress (requires e2compr package)\n");
19     fpe("    d   Do not backup with dump(8)\n");
20     fpe("    D   Synchronous directory updates\n");
21     fpe("    i   Immutable (privilege required)\n");
22     fpe("    j   Enable ext3/ext4 data journaling\n");
23     fpe("    s   Secure deletion (not implemented)\n");
24     fpe("    S   Synchronous file updates\n");
25     fpe("    t   Disable tail-packing (Reiserfs only)\n");
26     fpe("    T   Mark as top-level directory for Orlov algorithm\n");
27     fpe("    u   Undelete (not implemented)\n");
28     exit(EXIT_FAILURE);
29 }
30
31 int
32 main(int argc, char *argv[])
33 {
34     int attr, oldAttr, j, fd;
35     char *p;
36
37     if (argc < 3 || strchr("+-=", argv[1][0]) == NULL ||
38             strcmp(argv[1], "--help") == 0)
39         usageError(argv[0]);
40
41     /* Build bit mask based on attribute string in argv[1] */
42
43     attr = 0;
44     for (p = &argv[1][1]; *p != '\0'; p++) {
45         switch (*p) {
46         case 'a': attr |= FS_APPEND_FL;         break;
47         case 'A': attr |= FS_NOATIME_FL;        break;
48         case 'c': attr |= FS_COMPR_FL;          break;
49         case 'd': attr |= FS_NODUMP_FL;         break;
50         case 'D': attr |= FS_DIRSYNC_FL;        break;
51         case 'i': attr |= FS_IMMUTABLE_FL;      break;
52         case 'j': attr |= FS_JOURNAL_DATA_FL;   break;
53         case 's': attr |= FS_SECRM_FL;          break;
54         case 'S': attr |= FS_SYNC_FL;           break;
55         case 't': attr |= FS_NOTAIL_FL;         break;
56         case 'T': attr |= FS_TOPDIR_FL;         break;
57         case 'u': attr |= FS_UNRM_FL;           break;
58         default:  usageError(argv[0]);
59         }
60     }
61
62     /* Open each filename in turn, and change its attributes */
63
64     for (j = 2; j < argc; j++) {
65         fd = open(argv[j], O_RDONLY);
66         if (fd == -1) {         /* Most likely error is nonexistent file */
67             errMsg("open: %s", argv[j]);
68             continue;
69         }
70
71         /* If argv[1] starts with + or -, then retrieve existing
72            attribute bit mask and modify as required */
73
74         if (argv[1][0] == '+' || argv[1][0] == '-') {
75             if (ioctl(fd, FS_IOC_GETFLAGS, &oldAttr) == -1)
76                 errExit("ioctl1: %s", argv[j]);
77             attr = (*argv[1] == '-') ? (oldAttr & ~attr) : (oldAttr | attr);
78         }
79
80         if (ioctl(fd, FS_IOC_SETFLAGS, &attr) == -1)
81             errExit("ioctl2: %s", argv[j]);
82         if (close(fd) == -1)
83             errExit("close");
84     }
85
86     exit(EXIT_SUCCESS);
87 }
