/*************************************************************************\
*                  Copyright (C) Michael Kerrisk, 2014.                   *
*                                                                         *
* This program is free software. You may use, modify, and redistribute it *
* under the terms of the GNU Affero General Public License as published   *
* by the Free Software Foundation, either version 3 or (at your option)   *
* any later version. This program is distributed without any warranty.    *
* See the file COPYING.agpl-v3 for details.                               *
\*************************************************************************/

/* Listing 15-2 */

 1 #include <pwd.h>
 2 #include <grp.h>
 3 #include "ugid_functions.h"             /* Declarations of userIdFromName()
 4                                            and groupIdFromName() */
 5 #include "tlpi_hdr.h"
 6
 7 int
 8 main(int argc, char *argv[])
 9 {
10     uid_t uid;
11     gid_t gid;
12     int j;
13     Boolean errFnd;
14
15     if (argc < 3 || strcmp(argv[1], "--help") == 0)
16         usageErr("%s owner group [file...]\n"
17                 "        owner or group can be '-', "
18                 "meaning leave unchanged\n", argv[0]);
19
20     if (strcmp(argv[1], "-") == 0) {            /* "-" ==> don't change owner */
21         uid = -1;
22     } else {                                    /* Turn user name into UID */
23         uid = userIdFromName(argv[1]);
24         if (uid == -1)
25             fatal("No such user (%s)", argv[1]);
26     }
27
28     if (strcmp(argv[2], "-") == 0) {            /* "-" ==> don't change group */
29         gid = -1;
30     } else {                                    /* Turn group name into GID */
31         gid = groupIdFromName(argv[2]);
32         if (gid == -1)
33             fatal("No group user (%s)", argv[1]);
34     }
35
36     /* Change ownership of all files named in remaining arguments */
37
38     errFnd = FALSE;
39     for (j = 3; j < argc; j++) {
40         if (chown(argv[j], uid, gid) == -1) {
41             errMsg("chown: %s", argv[j]);
42             errFnd = TRUE;
43         }
44     }
45
46     exit(errFnd ? EXIT_FAILURE : EXIT_SUCCESS);
47 }
