/*************************************************************************\
*                  Copyright (C) Michael Kerrisk, 2014.                   *
*                                                                         *
* This program is free software. You may use, modify, and redistribute it *
* under the terms of the GNU Affero General Public License as published   *
* by the Free Software Foundation, either version 3 or (at your option)   *
* any later version. This program is distributed without any warranty.    *
* See the file COPYING.agpl-v3 for details.                               *
\*************************************************************************/

/* Listing 14-1 */

 1 #include <sys/mount.h>
 2 #include "tlpi_hdr.h"
 3
 4 static void
 5 usageError(const char *progName, const char *msg)
 6 {
 7     if (msg != NULL)
 8         fprintf(stderr, "%s", msg);
 9
10     fprintf(stderr, "Usage: %s [options] source target\n\n", progName);
11     fprintf(stderr, "Available options:\n");
12 #define fpe(str) fprintf(stderr, "    " str)    /* Shorter! */
13     fpe("-t fstype        [e.g., 'ext2' or 'reiserfs']\n");
14     fpe("-o data          [file system-dependent options,\n");
15     fpe("                 e.g., 'bsdgroups' for ext2]\n");
16     fpe("-f mountflags    can include any of:\n");
17 #define fpe2(str) fprintf(stderr, "            " str)
18     fpe2("b - MS_BIND         create a bind mount\n");
19     fpe2("d - MS_DIRSYNC      synchronous directory updates\n");
20     fpe2("l - MS_MANDLOCK     permit mandatory locking\n");
21     fpe2("m - MS_MOVE         atomically move subtree\n");
22     fpe2("A - MS_NOATIME      don't update atime (last access time)\n");
23     fpe2("V - MS_NODEV        don't permit device access\n");
24     fpe2("D - MS_NODIRATIME   don't update atime on directories\n");
25     fpe2("E - MS_NOEXEC       don't allow executables\n");
26     fpe2("S - MS_NOSUID       disable set-user/group-ID programs\n");
27     fpe2("r - MS_RDONLY       read-only mount\n");
28     fpe2("c - MS_REC          recursive mount\n");
29     fpe2("R - MS_REMOUNT      remount\n");
30     fpe2("s - MS_SYNCHRONOUS  make writes synchronous\n");
31     exit(EXIT_FAILURE);
32 }
33
34 int
35 main(int argc, char *argv[])
36 {
37     unsigned long flags;
38     char *data, *fstype;
39     int j, opt;
40
41     flags = 0;
42     data = NULL;
43     fstype = NULL;
44
45     while ((opt = getopt(argc, argv, "o:t:f:")) != -1) {
46         switch (opt) {
47         case 'o':
48             data = optarg;
49             break;
50
51         case 't':
52             fstype = optarg;
53             break;
54
55         case 'f':
56             for (j = 0; j < strlen(optarg); j++) {
57                 switch (optarg[j]) {
58                 case 'b': flags |= MS_BIND;             break;
59                 case 'd': flags |= MS_DIRSYNC;          break;
60                 case 'l': flags |= MS_MANDLOCK;         break;
61                 case 'm': flags |= MS_MOVE;             break;
62                 case 'A': flags |= MS_NOATIME;          break;
63                 case 'V': flags |= MS_NODEV;            break;
64                 case 'D': flags |= MS_NODIRATIME;       break;
65                 case 'E': flags |= MS_NOEXEC;           break;
66                 case 'S': flags |= MS_NOSUID;           break;
67                 case 'r': flags |= MS_RDONLY;           break;
68                 case 'c': flags |= MS_REC;              break;
69                 case 'R': flags |= MS_REMOUNT;          break;
70                 case 's': flags |= MS_SYNCHRONOUS;      break;
71                 default:  usageError(argv[0], NULL);
72                 }
73             }
74             break;
75
76         default:
77             usageError(argv[0], NULL);
78         }
79     }
80
81     if (argc != optind + 2)
82         usageError(argv[0], "Wrong number of arguments\n");
83
84     if (mount(argv[optind], argv[optind + 1], fstype, flags, data) == -1)
85         errExit("mount");
86
87     exit(EXIT_SUCCESS);
88 }
