/*************************************************************************\
*                  Copyright (C) Michael Kerrisk, 2014.                   *
*                                                                         *
* This program is free software. You may use, modify, and redistribute it *
* under the terms of the GNU Affero General Public License as published   *
* by the Free Software Foundation, either version 3 or (at your option)   *
* any later version. This program is distributed without any warranty.    *
* See the file COPYING.agpl-v3 for details.                               *
\*************************************************************************/

/* Listing 19-1 */

 1 #include <sys/inotify.h>
 2 #include <limits.h>
 3 #include "tlpi_hdr.h"
 4
 5 static void             /* Display information from inotify_event structure */
 6 displayInotifyEvent(struct inotify_event *i)
 7 {
 8     printf("    wd =%2d; ", i->wd);
 9     if (i->cookie > 0)
10         printf("cookie =%4d; ", i->cookie);
11
12     printf("mask = ");
13     if (i->mask & IN_ACCESS)        printf("IN_ACCESS ");
14     if (i->mask & IN_ATTRIB)        printf("IN_ATTRIB ");
15     if (i->mask & IN_CLOSE_NOWRITE) printf("IN_CLOSE_NOWRITE ");
16     if (i->mask & IN_CLOSE_WRITE)   printf("IN_CLOSE_WRITE ");
17     if (i->mask & IN_CREATE)        printf("IN_CREATE ");
18     if (i->mask & IN_DELETE)        printf("IN_DELETE ");
19     if (i->mask & IN_DELETE_SELF)   printf("IN_DELETE_SELF ");
20     if (i->mask & IN_IGNORED)       printf("IN_IGNORED ");
21     if (i->mask & IN_ISDIR)         printf("IN_ISDIR ");
22     if (i->mask & IN_MODIFY)        printf("IN_MODIFY ");
23     if (i->mask & IN_MOVE_SELF)     printf("IN_MOVE_SELF ");
24     if (i->mask & IN_MOVED_FROM)    printf("IN_MOVED_FROM ");
25     if (i->mask & IN_MOVED_TO)      printf("IN_MOVED_TO ");
26     if (i->mask & IN_OPEN)          printf("IN_OPEN ");
27     if (i->mask & IN_Q_OVERFLOW)    printf("IN_Q_OVERFLOW ");
28     if (i->mask & IN_UNMOUNT)       printf("IN_UNMOUNT ");
29     printf("\n");
30
31     if (i->len > 0)
32         printf("        name = %s\n", i->name);
33 }
34
35 #define BUF_LEN (10 * (sizeof(struct inotify_event) + NAME_MAX + 1))
36
37 int
38 main(int argc, char *argv[])
39 {
40     int inotifyFd, wd, j;
41     char buf[BUF_LEN] __attribute__ ((aligned(8)));
42     ssize_t numRead;
43     char *p;
44     struct inotify_event *event;
45
46     if (argc < 2 || strcmp(argv[1], "--help") == 0)
47         usageErr("%s pathname...\n", argv[0]);
48
49     inotifyFd = inotify_init();                 /* Create inotify instance */
50     if (inotifyFd == -1)
51         errExit("inotify_init");
52
53     for (j = 1; j < argc; j++) {
54         wd = inotify_add_watch(inotifyFd, argv[j], IN_ALL_EVENTS);
55         if (wd == -1)
56             errExit("inotify_add_watch");
57
58         printf("Watching %s using wd %d\n", argv[j], wd);
59     }
60
61     for (;;) {                                  /* Read events forever */
62         numRead = read(inotifyFd, buf, BUF_LEN);
63         if (numRead == 0)
64             fatal("read() from inotify fd returned 0!");
65
66         if (numRead == -1)
67             errExit("read");
68
69         printf("Read %ld bytes from inotify fd\n", (long) numRead);
70
71         /* Process all of the events in buffer returned by read() */
72
73         for (p = buf; p < buf + numRead; ) {
74             event = (struct inotify_event *) p;
75             displayInotifyEvent(event);
76
77             p += sizeof(struct inotify_event) + event->len;
78         }
79     }
80
81     exit(EXIT_SUCCESS);
82 }
