 1 /*************************************************************************\
 2 *                  Copyright (C) Michael Kerrisk, 2014.                   *
 3 *                                                                         *
 4 * This program is free software. You may use, modify, and redistribute it *
 5 * under the terms of the GNU Lesser General Public License as published   *
 6 * by the Free Software Foundation, either version 3 or (at your option)   *
 7 * any later version. This program is distributed without any warranty.    *
 8 * See the files COPYING.lgpl-v3 and COPYING.gpl-v3 for details.           *
 9 \*************************************************************************/
10
11 /* Listing 26-2 */
12
13  1 #define _GNU_SOURCE     /* Get strsignal() declaration from <string.h> */
14  2 #include <string.h>
15  3 #include <sys/wait.h>
16  4 #include "print_wait_status.h"  /* Declaration of printWaitStatus() */
17  5 #include "tlpi_hdr.h"
18  6
19  7 /* NOTE: The following function employs printf(), which is not
20  8    async-signal-safe (see Section 21.1.2). As such, this function is
21  9    also not async-signal-safe (i.e., beware of calling it from a
22 10    SIGCHLD handler). */
23 11
24 12 void                    /* Examine a wait() status using the W* macros */
25 13 printWaitStatus(const char *msg, int status)
26 14 {
27 15     if (msg != NULL)
28 16         printf("%s", msg);
29 17
30 18     if (WIFEXITED(status)) {
31 19         printf("child exited, status=%d\n", WEXITSTATUS(status));
32 20
33 21     } else if (WIFSIGNALED(status)) {
34 22         printf("child killed by signal %d (%s)",
35 23                 WTERMSIG(status), strsignal(WTERMSIG(status)));
36 24 #ifdef WCOREDUMP        /* Not in SUSv3, may be absent on some systems */
37 25         if (WCOREDUMP(status))
38 26             printf(" (core dumped)");
39 27 #endif
40 28         printf("\n");
41 29
42 30     } else if (WIFSTOPPED(status)) {
43 31         printf("child stopped by signal %d (%s)\n",
44 32                 WSTOPSIG(status), strsignal(WSTOPSIG(status)));
45 33
46 34 #ifdef WIFCONTINUED     /* SUSv3 has this, but older Linux versions and
47 35                            some other UNIX implementations don't */
48 36     } else if (WIFCONTINUED(status)) {
49 37         printf("child continued\n");
50 38 #endif
51 39
52 40     } else {            /* Should never happen */
53 41         printf("what happened to this child? (status=%x)\n",
54 42                 (unsigned int) status);
55 43     }
56 44 }
