/*************************************************************************\
*                  Copyright (C) Michael Kerrisk, 2014.                   *
*                                                                         *
* This program is free software. You may use, modify, and redistribute it *
* under the terms of the GNU Affero General Public License as published   *
* by the Free Software Foundation, either version 3 or (at your option)   *
* any later version. This program is distributed without any warranty.    *
* See the file COPYING.agpl-v3 for details.                               *
\*************************************************************************/

/* Listing 40-3 */

 1 #define _GNU_SOURCE
 2 #include <time.h>
 3 #include <utmpx.h>
 4 #include <paths.h>              /* Definitions of _PATH_UTMP and _PATH_WTMP */
 5 #include "tlpi_hdr.h"
 6
 7 int
 8 main(int argc, char *argv[])
 9 {
10     struct utmpx ut;
11     char *devName;
12
13     if (argc < 2 || strcmp(argv[1], "--help") == 0)
14         usageErr("%s username [sleep-time]\n", argv[0]);
15
16     /* Initialize login record for utmp and wtmp files */
17
18     memset(&ut, 0, sizeof(struct utmpx));
19     ut.ut_type = USER_PROCESS;          /* This is a user login */
20     strncpy(ut.ut_user, argv[1], sizeof(ut.ut_user));
21     if (time((time_t *) &ut.ut_tv.tv_sec) == -1)
22         errExit("time");                /* Stamp with current time */
23     ut.ut_pid = getpid();
24
25     /* Set ut_line and ut_id based on the terminal associated with
26        'stdin'. This code assumes terminals named "/dev/[pt]t[sy]*".
27        The "/dev/" dirname is 5 characters; the "[pt]t[sy]" filename
28        prefix is 3 characters (making 8 characters in all). */
29
30     devName = ttyname(STDIN_FILENO);
31     if (devName == NULL)
32         errExit("ttyname");
33     if (strlen(devName) <= 8)           /* Should never happen */
34         fatal("Terminal name is too short: %s", devName);
35
36     strncpy(ut.ut_line, devName + 5, sizeof(ut.ut_line));
37     strncpy(ut.ut_id, devName + 8, sizeof(ut.ut_id));
38
39     printf("Creating login entries in utmp and wtmp\n");
40     printf("        using pid %ld, line %.*s, id %.*s\n",
41             (long) ut.ut_pid, (int) sizeof(ut.ut_line), ut.ut_line,
42             (int) sizeof(ut.ut_id), ut.ut_id);
43
44     setutxent();                        /* Rewind to start of utmp file */
45     if (pututxline(&ut) == NULL)        /* Write login record to utmp */
46         errExit("pututxline");
47     updwtmpx(_PATH_WTMP, &ut);          /* Append login record to wtmp */
48
49     /* Sleep a while, so we can examine utmp and wtmp files */
50
51     sleep((argc > 2) ? getInt(argv[2], GN_NONNEG, "sleep-time") : 15);
52
53     /* Now do a "logout"; use values from previously initialized 'ut',
54        except for changes below */
55
56     ut.ut_type = DEAD_PROCESS;          /* Required for logout record */
57     time((time_t *) &ut.ut_tv.tv_sec);  /* Stamp with logout time */
58     memset(&ut.ut_user, 0, sizeof(ut.ut_user));
59                                         /* Logout record has null username */
60
61     printf("Creating logout entries in utmp and wtmp\n");
62     setutxent();                        /* Rewind to start of utmp file */
63     if (pututxline(&ut) == NULL)        /* Overwrite previous utmp record */
64         errExit("pututxline");
65     updwtmpx(_PATH_WTMP, &ut);          /* Append logout record to wtmp */
66
67     endutxent();
68     exit(EXIT_SUCCESS);
69 }
