/*************************************************************************\
*                  Copyright (C) Michael Kerrisk, 2014.                   *
*                                                                         *
* This program is free software. You may use, modify, and redistribute it *
* under the terms of the GNU Affero General Public License as published   *
* by the Free Software Foundation, either version 3 or (at your option)   *
* any later version. This program is distributed without any warranty.    *
* See the file COPYING.agpl-v3 for details.                               *
\*************************************************************************/

/* Listing 40-4 */

 1 #include <time.h>
 2 #include <lastlog.h>
 3 #include <paths.h>                      /* Definition of _PATH_LASTLOG */
 4 #include <fcntl.h>
 5 #include "ugid_functions.h"             /* Declaration of userIdFromName() */
 6 #include "tlpi_hdr.h"
 7
 8 int
 9 main(int argc, char *argv[])
10 {
11     struct lastlog llog;
12     int fd, j;
13     uid_t uid;
14
15     if (argc > 1 && strcmp(argv[1], "--help") == 0)
16         usageErr("%s [username...]\n", argv[0]);
17
18     fd = open(_PATH_LASTLOG, O_RDONLY);
19     if (fd == -1)
20         errExit("open");
21
22     for (j = 1; j < argc; j++) {
23         uid = userIdFromName(argv[j]);
24         if (uid == -1) {
25             printf("No such user: %s\n", argv[j]);
26             continue;
27         }
28
29         if (lseek(fd, uid * sizeof(struct lastlog), SEEK_SET) == -1)
30             errExit("lseek");
31
32         if (read(fd, &llog, sizeof(struct lastlog)) <= 0) {
33             printf("read failed for %s\n", argv[j]);    /* EOF or error */
34             continue;
35         }
36
37         printf("%-8.8s %-6.6s %-20.20s %s", argv[j], llog.ll_line,
38                 llog.ll_host, ctime((time_t *) &llog.ll_time));
39     }
40
41     close(fd);
42     exit(EXIT_SUCCESS);
43 }
