/*************************************************************************\
*                  Copyright (C) Michael Kerrisk, 2014.                   *
*                                                                         *
* This program is free software. You may use, modify, and redistribute it *
* under the terms of the GNU Affero General Public License as published   *
* by the Free Software Foundation, either version 3 or (at your option)   *
* any later version. This program is distributed without any warranty.    *
* See the file COPYING.agpl-v3 for details.                               *
\*************************************************************************/

/* Listing 7-1 */

 1 #define _BSD_SOURCE
 2 #include "tlpi_hdr.h"
 3
 4 #define MAX_ALLOCS 1000000
 5
 6 int
 7 main(int argc, char *argv[])
 8 {
 9     char *ptr[MAX_ALLOCS];
10     int freeStep, freeMin, freeMax, blockSize, numAllocs, j;
11
12     printf("\n");
13
14     if (argc < 3 || strcmp(argv[1], "--help") == 0)
15         usageErr("%s num-allocs block-size [step [min [max]]]\n", argv[0]);
16
17     numAllocs = getInt(argv[1], GN_GT_0, "num-allocs");
18     if (numAllocs > MAX_ALLOCS)
19         cmdLineErr("num-allocs > %d\n", MAX_ALLOCS);
20
21     blockSize = getInt(argv[2], GN_GT_0 | GN_ANY_BASE, "block-size");
22
23     freeStep = (argc > 3) ? getInt(argv[3], GN_GT_0, "step") : 1;
24     freeMin =  (argc > 4) ? getInt(argv[4], GN_GT_0, "min") : 1;
25     freeMax =  (argc > 5) ? getInt(argv[5], GN_GT_0, "max") : numAllocs;
26
27     if (freeMax > numAllocs)
28         cmdLineErr("free-max > num-allocs\n");
29
30     printf("Initial program break:          %10p\n", sbrk(0));
31
32     printf("Allocating %d*%d bytes\n", numAllocs, blockSize);
33     for (j = 0; j < numAllocs; j++) {
34         ptr[j] = malloc(blockSize);
35         if (ptr[j] == NULL)
36             errExit("malloc");
37     }
38
39     printf("Program break is now:           %10p\n", sbrk(0));
40
41     printf("Freeing blocks from %d to %d in steps of %d\n",
42                 freeMin, freeMax, freeStep);
43     for (j = freeMin - 1; j < freeMax; j += freeStep)
44         free(ptr[j]);
45
46     printf("After free(), program break is: %10p\n", sbrk(0));
47
48     exit(EXIT_SUCCESS);
49 }
