/*************************************************************************\
*                  Copyright (C) Michael Kerrisk, 2014.                   *
*                                                                         *
* This program is free software. You may use, modify, and redistribute it *
* under the terms of the GNU Affero General Public License as published   *
* by the Free Software Foundation, either version 3 or (at your option)   *
* any later version. This program is distributed without any warranty.    *
* See the file COPYING.agpl-v3 for details.                               *
\*************************************************************************/

/* Listing 34-4 */

 1 #define _GNU_SOURCE     /* Get strsignal() declaration from <string.h> */
 2 #include <string.h>
 3 #include <signal.h>
 4 #include "tlpi_hdr.h"
 5
 6 static void             /* Handler for SIGHUP */
 7 handler(int sig)
 8 {
 9     printf("PID %ld: caught signal %2d (%s)\n", (long) getpid(),
10             sig, strsignal(sig));
11                         /* UNSAFE (see Section 21.1.2) */
12 }
13
14 int
15 main(int argc, char *argv[])
16 {
17     pid_t parentPid, childPid;
18     int j;
19     struct sigaction sa;
20
21     if (argc < 2 || strcmp(argv[1], "--help") == 0)
22         usageErr("%s {d|s}... [ > sig.log 2>&1 ]\n", argv[0]);
23
24     setbuf(stdout, NULL);               /* Make stdout unbuffered */
25
26     parentPid = getpid();
27     printf("PID of parent process is:       %ld\n", (long) parentPid);
28     printf("Foreground process group ID is: %ld\n",
29             (long) tcgetpgrp(STDIN_FILENO));
30
31     for (j = 1; j < argc; j++) {        /* Create child processes */
32         childPid = fork();
33         if (childPid == -1)
34             errExit("fork");
35
36         if (childPid == 0) {            /* If child... */
37             if (argv[j][0] == 'd')      /* 'd' --> to different pgrp */
38                 if (setpgid(0, 0) == -1)
39                     errExit("setpgid");
40
41             sigemptyset(&sa.sa_mask);
42             sa.sa_flags = 0;
43             sa.sa_handler = handler;
44             if (sigaction(SIGHUP, &sa, NULL) == -1)
45                 errExit("sigaction");
46             break;                      /* Child exits loop */
47         }
48     }
49
50     /* All processes fall through to here */
51
52     alarm(60);      /* Ensure each process eventually terminates */
53
54     printf("PID=%ld PGID=%ld\n", (long) getpid(), (long) getpgrp());
55     for (;;)
56         pause();        /* Wait for signals */
57 }
