/*************************************************************************\
*                  Copyright (C) Michael Kerrisk, 2014.                   *
*                                                                         *
* This program is free software. You may use, modify, and redistribute it *
* under the terms of the GNU Affero General Public License as published   *
* by the Free Software Foundation, either version 3 or (at your option)   *
* any later version. This program is distributed without any warranty.    *
* See the file COPYING.agpl-v3 for details.                               *
\*************************************************************************/

/* Listing 34-6 */

 1 #include <signal.h>
 2 #include "tlpi_hdr.h"
 3
 4 static void                             /* Handler for SIGTSTP */
 5 tstpHandler(int sig)
 6 {
 7     sigset_t tstpMask, prevMask;
 8     int savedErrno;
 9     struct sigaction sa;
10
11     savedErrno = errno;                 /* In case we change 'errno' here */
12
13     printf("Caught SIGTSTP\n");         /* UNSAFE (see Section 21.1.2) */
14
15     if (signal(SIGTSTP, SIG_DFL) == SIG_ERR)
16         errExit("signal");              /* Set handling to default */
17
18     raise(SIGTSTP);                     /* Generate a further SIGTSTP */
19
20     /* Unblock SIGTSTP; the pending SIGTSTP immediately suspends the program */
21
22     sigemptyset(&tstpMask);
23     sigaddset(&tstpMask, SIGTSTP);
24     if (sigprocmask(SIG_UNBLOCK, &tstpMask, &prevMask) == -1)
25         errExit("sigprocmask");
26
27     /* Execution resumes here after SIGCONT */
28
29     if (sigprocmask(SIG_SETMASK, &prevMask, NULL) == -1)
30         errExit("sigprocmask");         /* Reblock SIGTSTP */
31
32     sigemptyset(&sa.sa_mask);           /* Reestablish handler */
33     sa.sa_flags = SA_RESTART;
34     sa.sa_handler = tstpHandler;
35     if (sigaction(SIGTSTP, &sa, NULL) == -1)
36         errExit("sigaction");
37
38     printf("Exiting SIGTSTP handler\n");
39     errno = savedErrno;
40 }
41
42 int
43 main(int argc, char *argv[])
44 {
45     struct sigaction sa;
46
47     /* Only establish handler for SIGTSTP if it is not being ignored */
48
49     if (sigaction(SIGTSTP, NULL, &sa) == -1)
50         errExit("sigaction");
51
52     if (sa.sa_handler != SIG_IGN) {
53         sigemptyset(&sa.sa_mask);
54         sa.sa_flags = SA_RESTART;
55         sa.sa_handler = tstpHandler;
56         if (sigaction(SIGTSTP, &sa, NULL) == -1)
57             errExit("sigaction");
58     }
59
60     for (;;) {                          /* Wait for signals */
61         pause();
62         printf("Main\n");
63     }
64 }
