/*************************************************************************\
*                  Copyright (C) Michael Kerrisk, 2014.                   *
*                                                                         *
* This program is free software. You may use, modify, and redistribute it *
* under the terms of the GNU Affero General Public License as published   *
* by the Free Software Foundation, either version 3 or (at your option)   *
* any later version. This program is distributed without any warranty.    *
* See the file COPYING.agpl-v3 for details.                               *
\*************************************************************************/

/* Listing 34-7 */

 1 #define _GNU_SOURCE     /* Get declaration of strsignal() from <string.h> */
 2 #include <string.h>
 3 #include <signal.h>
 4 #include "tlpi_hdr.h"
 5
 6 static void             /* Signal handler */
 7 handler(int sig)
 8 {
 9     printf("PID=%ld: caught signal %d (%s)\n", (long) getpid(),
10             sig, strsignal(sig));       /* UNSAFE (see Section 21.1.2) */
11 }
12
13 int
14 main(int argc, char *argv[])
15 {
16     int j;
17     struct sigaction sa;
18
19     if (argc < 2 || strcmp(argv[1], "--help") == 0)
20         usageErr("%s {s|p} ...\n", argv[0]);
21
22     setbuf(stdout, NULL);               /* Make stdout unbuffered */
23
24     sigemptyset(&sa.sa_mask);
25     sa.sa_flags = 0;
26     sa.sa_handler = handler;
27     if (sigaction(SIGHUP, &sa, NULL) == -1)
28         errExit("sigaction");
29     if (sigaction(SIGCONT, &sa, NULL) == -1)
30         errExit("sigaction");
31
32     printf("parent: PID=%ld, PPID=%ld, PGID=%ld, SID=%ld\n",
33             (long) getpid(), (long) getppid(),
34             (long) getpgrp(), (long) getsid(0));
35
36     /* Create one child for each command-line argument */
37
38     for (j = 1; j < argc; j++) {
39         switch (fork()) {
40         case -1:
41             errExit("fork");
42
43         case 0:         /* Child */
44             printf("child:  PID=%ld, PPID=%ld, PGID=%ld, SID=%ld\n",
45                     (long) getpid(), (long) getppid(),
46                     (long) getpgrp(), (long) getsid(0));
47
48             if (argv[j][0] == 's') {    /* Stop via signal */
49                 printf("PID=%ld stopping\n", (long) getpid());
50                 raise(SIGSTOP);
51             } else {                    /* Wait for signal */
52                 alarm(60);              /* So we die if not SIGHUPed */
53                 printf("PID=%ld pausing\n", (long) getpid());
54                 pause();
55             }
56
57             _exit(EXIT_SUCCESS);
58
59         default:        /* Parent carries on round loop */
60             break;
61         }
62     }
63
64     /* Parent falls through to here after creating all children */
65
66     sleep(3);                           /* Give children a chance to start */
67     printf("parent exiting\n");
68     exit(EXIT_SUCCESS);                 /* And orphan them and their group */
69 }
