/*************************************************************************\
*                  Copyright (C) Michael Kerrisk, 2014.                   *
*                                                                         *
* This program is free software. You may use, modify, and redistribute it *
* under the terms of the GNU Affero General Public License as published   *
* by the Free Software Foundation, either version 3 or (at your option)   *
* any later version. This program is distributed without any warranty.    *
* See the file COPYING.agpl-v3 for details.                               *
\*************************************************************************/

/* Listing 44-7 */

 1 #include <signal.h>
 2 #include "fifo_seqnum.h"
 3
 4 int
 5 main(int argc, char *argv[])
 6 {
 7     int serverFd, dummyFd, clientFd;
 8     char clientFifo[CLIENT_FIFO_NAME_LEN];
 9     struct request req;
10     struct response resp;
11     int seqNum = 0;                     /* This is our "service" */
12
13     /* Create well-known FIFO, and open it for reading */
14
15     umask(0);                           /* So we get the permissions we want */
16     if (mkfifo(SERVER_FIFO, S_IRUSR | S_IWUSR | S_IWGRP) == -1
17             && errno != EEXIST)
18         errExit("mkfifo %s", SERVER_FIFO);
19     serverFd = open(SERVER_FIFO, O_RDONLY);
20     if (serverFd == -1)
21         errExit("open %s", SERVER_FIFO);
22
23     /* Open an extra write descriptor, so that we never see EOF */
24
25     dummyFd = open(SERVER_FIFO, O_WRONLY);
26     if (dummyFd == -1)
27         errExit("open %s", SERVER_FIFO);
28
29     if (signal(SIGPIPE, SIG_IGN) == SIG_ERR)
30         errExit("signal");
31
32     for (;;) {                          /* Read requests and send responses */
33         if (read(serverFd, &req, sizeof(struct request))
34                 != sizeof(struct request)) {
35             fprintf(stderr, "Error reading request; discarding\n");
36             continue;                   /* Either partial read or error */
37         }
38
39         /* Open client FIFO (previously created by client) */
40
41         snprintf(clientFifo, CLIENT_FIFO_NAME_LEN, CLIENT_FIFO_TEMPLATE,
42                 (long) req.pid);
43         clientFd = open(clientFifo, O_WRONLY);
44         if (clientFd == -1) {           /* Open failed, give up on client */
45             errMsg("open %s", clientFifo);
46             continue;
47         }
48
49         /* Send response and close FIFO */
50
51         resp.seqNum = seqNum;
52         if (write(clientFd, &resp, sizeof(struct response))
53                 != sizeof(struct response))
54             fprintf(stderr, "Error writing to FIFO %s\n", clientFifo);
55         if (close(clientFd) == -1)
56             errMsg("close");
57
58         seqNum += req.seqLen;           /* Update our sequence number */
59     }
60 }
