/*************************************************************************\
*                  Copyright (C) Michael Kerrisk, 2014.                   *
*                                                                         *
* This program is free software. You may use, modify, and redistribute it *
* under the terms of the GNU Affero General Public License as published   *
* by the Free Software Foundation, either version 3 or (at your option)   *
* any later version. This program is distributed without any warranty.    *
* See the file COPYING.agpl-v3 for details.                               *
\*************************************************************************/

/* Listing 44-3 */

 1 #include "curr_time.h"                      /* Declaration of currTime() */
 2 #include "tlpi_hdr.h"
 3
 4 int
 5 main(int argc, char *argv[])
 6 {
 7     int pfd[2];                             /* Process synchronization pipe */
 8     int j, dummy;
 9
10     if (argc < 2 || strcmp(argv[1], "--help") == 0)
11         usageErr("%s sleep-time...\n", argv[0]);
12
13     setbuf(stdout, NULL);                   /* Make stdout unbuffered, since we
14                                                terminate child with _exit() */
15     printf("%s  Parent started\n", currTime("%T"));
16
17     if (pipe(pfd) == -1)
18         errExit("pipe");
19
20     for (j = 1; j < argc; j++) {
21         switch (fork()) {
22         case -1:
23             errExit("fork %d", j);
24
25         case 0: /* Child */
26             if (close(pfd[0]) == -1)        /* Read end is unused */
27                 errExit("close");
28
29             /* Child does some work, and lets parent know it's done */
30
31             sleep(getInt(argv[j], GN_NONNEG, "sleep-time"));
32                                             /* Simulate processing */
33             printf("%s  Child %d (PID=%ld) closing pipe\n",
34                     currTime("%T"), j, (long) getpid());
35             if (close(pfd[1]) == -1)
36                 errExit("close");
37
38             /* Child now carries on to do other things... */
39
40             _exit(EXIT_SUCCESS);
41
42         default: /* Parent loops to create next child */
43             break;
44         }
45     }
46
47     /* Parent comes here; close write end of pipe so we can see EOF */
48
49     if (close(pfd[1]) == -1)                /* Write end is unused */
50         errExit("close");
51
52     /* Parent may do other work, then synchronizes with children */
53
54     if (read(pfd[0], &dummy, 1) != 0)
55         fatal("parent didn't get EOF");
56     printf("%s  Parent ready to go\n", currTime("%T"));
57
58     /* Parent can now carry on to do other things... */
59
60     exit(EXIT_SUCCESS);
61 }
