/*************************************************************************\
*                  Copyright (C) Michael Kerrisk, 2014.                   *
*                                                                         *
* This program is free software. You may use, modify, and redistribute it *
* under the terms of the GNU Affero General Public License as published   *
* by the Free Software Foundation, either version 3 or (at your option)   *
* any later version. This program is distributed without any warranty.    *
* See the file COPYING.agpl-v3 for details.                               *
\*************************************************************************/

/* Listing 52-6 */

 1 #include <signal.h>
 2 #include <mqueue.h>
 3 #include <fcntl.h>              /* For definition of O_NONBLOCK */
 4 #include "tlpi_hdr.h"
 5
 6 #define NOTIFY_SIG SIGUSR1
 7
 8 static void
 9 handler(int sig)
10 {
11     /* Just interrupt sigsuspend() */
12 }
13
14 int
15 main(int argc, char *argv[])
16 {
17     struct sigevent sev;
18     mqd_t mqd;
19     struct mq_attr attr;
20     void *buffer;
21     ssize_t numRead;
22     sigset_t blockMask, emptyMask;
23     struct sigaction sa;
24
25     if (argc != 2 || strcmp(argv[1], "--help") == 0)
26         usageErr("%s mq-name\n", argv[0]);
27
28     mqd = mq_open(argv[1], O_RDONLY | O_NONBLOCK);
29     if (mqd == (mqd_t) -1)
30         errExit("mq_open");
31
32     if (mq_getattr(mqd, &attr) == -1)
33         errExit("mq_getattr");
34
35     buffer = malloc(attr.mq_msgsize);
36     if (buffer == NULL)
37         errExit("malloc");
38
39     sigemptyset(&blockMask);
40     sigaddset(&blockMask, NOTIFY_SIG);
41     if (sigprocmask(SIG_BLOCK, &blockMask, NULL) == -1)
42         errExit("sigprocmask");
43
44     sigemptyset(&sa.sa_mask);
45     sa.sa_flags = 0;
46     sa.sa_handler = handler;
47     if (sigaction(NOTIFY_SIG, &sa, NULL) == -1)
48         errExit("sigaction");
49
50     sev.sigev_notify = SIGEV_SIGNAL;
51     sev.sigev_signo = NOTIFY_SIG;
52     if (mq_notify(mqd, &sev) == -1)
53         errExit("mq_notify");
54
55     sigemptyset(&emptyMask);
56
57     for (;;) {
58         sigsuspend(&emptyMask);         /* Wait for notification signal */
59
60         if (mq_notify(mqd, &sev) == -1)
61             errExit("mq_notify");
62
63         while ((numRead = mq_receive(mqd, buffer, attr.mq_msgsize, NULL)) >= 0)
64             printf("Read %ld bytes\n", (long) numRead);
65
66         if (errno != EAGAIN)            /* Unexpected error */
67             errExit("mq_receive");
68     }
69 }
