/*************************************************************************\
*                  Copyright (C) Michael Kerrisk, 2014.                   *
*                                                                         *
* This program is free software. You may use, modify, and redistribute it *
* under the terms of the GNU Affero General Public License as published   *
* by the Free Software Foundation, either version 3 or (at your option)   *
* any later version. This program is distributed without any warranty.    *
* See the file COPYING.agpl-v3 for details.                               *
\*************************************************************************/

/* Listing 52-4 */

 1 #include <mqueue.h>
 2 #include <fcntl.h>              /* For definition of O_NONBLOCK */
 3 #include "tlpi_hdr.h"
 4
 5 static void
 6 usageError(const char *progName)
 7 {
 8     fprintf(stderr, "Usage: %s [-n] mq-name msg [prio]\n", progName);
 9     fprintf(stderr, "    -n           Use O_NONBLOCK flag\n");
10     exit(EXIT_FAILURE);
11 }
12
13 int
14 main(int argc, char *argv[])
15 {
16     int flags, opt;
17     mqd_t mqd;
18     unsigned int prio;
19
20     flags = O_WRONLY;
21     while ((opt = getopt(argc, argv, "n")) != -1) {
22         switch (opt) {
23         case 'n':   flags |= O_NONBLOCK;        break;
24         default:    usageError(argv[0]);
25         }
26     }
27
28     if (optind + 1 >= argc)
29         usageError(argv[0]);
30
31     mqd = mq_open(argv[optind], flags);
32     if (mqd == (mqd_t) -1)
33         errExit("mq_open");
34
35     prio = (argc > optind + 2) ? atoi(argv[optind + 2]) : 0;
36
37     if (mq_send(mqd, argv[optind + 1], strlen(argv[optind + 1]), prio) == -1)
38         errExit("mq_send");
39     exit(EXIT_SUCCESS);
40 }
