/*************************************************************************\
*                  Copyright (C) Michael Kerrisk, 2014.                   *
*                                                                         *
* This program is free software. You may use, modify, and redistribute it *
* under the terms of the GNU Affero General Public License as published   *
* by the Free Software Foundation, either version 3 or (at your option)   *
* any later version. This program is distributed without any warranty.    *
* See the file COPYING.agpl-v3 for details.                               *
\*************************************************************************/

/* Solution for Exercise 27-2 */

  1
  2 #include <stdio.h>
  3 #include <stdlib.h>
  4 #include <errno.h>
  5 #include <stdarg.h>
  6 #include <string.h>
  7 #include <unistd.h>
  8
  9 extern char **environ;
 10
 11 #define max(x,y) ((x) > (y) ? (x) : (y))
 12 #define SHELL_PATH "/bin/sh"            /* pathname for the standard shell */
 13
 14 /* Exec a script file using the standard shell */
 15
 16 static void
 17 execShScript(int argc, char *argv[], char *envp[])
 18 {
 19     char *shArgv[argc + 1];
 20     int j;
 21
 22     shArgv[0] = SHELL_PATH;
 23     for (j = 0; j <= argc; j++)
 24         shArgv[j + 1] = argv[j];
 25     execve(SHELL_PATH, shArgv, envp);
 26
 27     /* Only get here, if execve() fails, in which case we return
 28        to our caller. */
 29 }
 30
 31 int
 32 execlp(const char *filename, const char *arg, ...)
 33 {
 34     char **argv;                /* Argument vector for new program */
 35     int argc;                   /* Number of items used in argv */
 36     int argvSize;               /* Currently allocated size of argv */
 37     va_list argList;            /* For variable argument list parsing */
 38     char **envp;                /* Environment for new program */
 39     char *PATH;                 /* Value of PATH environment variable */
 40     char *pathname;             /* Path prefix + '/' + filename */
 41     char *prStart, *prEnd;      /* Start and end of prefix currently
 42                                    being processed from PATH */
 43     int savedErrno;
 44     int morePrefixes;           /* True if there are more prefixes in PATH */
 45     char *p;
 46     int j;
 47     int fndEACCES;              /* True if any execve() returned EACCES */
 48
 49     fndEACCES = 0;
 50
 51     /***** Build argument vector from variable length argument list *****/
 52
 53     argvSize = 100;
 54     argv = calloc(argvSize, sizeof(void *));
 55     if (argv == NULL) return -1;
 56
 57     argv[0] = (char *) arg;
 58     argc = 1;
 59
 60     /* Walk variable length argument list until NULL terminator is found,
 61        building argv as we go. */
 62
 63     va_start(argList, arg);
 64     while (argv[argc - 1] != NULL) {
 65         if (argc + 1 >= argvSize) {     /* Resize argv if required */
 66             argvSize += 100;
 67             argv = realloc(argv, sizeof(void *));
 68             if (argv == NULL) return -1;
 69         }
 70
 71         argv[argc] = va_arg(argList, char *);
 72         argc++;
 73     } 
 74
 75     va_end(argList);
 76
 77     /***** Use caller's environment to create envp vector *****/
 78
 79     for (j = 0; environ[j] != NULL; )   /* Calculate size of environ */
 80         j++;
 81     envp = calloc(sizeof(void *), j + 1);
 82     if (envp == NULL) return -1;
 83
 84     for (j = 0; environ[j] != NULL; j++)    /* Duplicate environ in envp */
 85         envp[j] = strdup(environ[j]);
 86     envp[j] = NULL;             /* List must be terminated by NULL pointer */
 87
 88     /***** Now try to exec filename *****/
 89
 90     if (strchr(filename, '/') != NULL) {
 91
 92          /* If file contains a slash, it's a pathname and we don't do
 93             a search using PATH */
 94
 95         pathname = strdup(filename);
 96
 97         execve(pathname, argv, envp);
 98
 99         savedErrno = errno;             /* So we can return correct errno */
100         if (errno == ENOEXEC)
101             execShScript(argc, argv, envp);
102
103         free(pathname);                 /* Avoid memory leaks */
104
105     } else {            /* Use PATH */
106
107         /* Treat undefined PATH as "." */
108
109         p = getenv("PATH");
110         PATH = (p == NULL || strlen(p) == 0) ? strdup(".") : strdup(p);
111
112         /* For each prefix in PATH, try to exec 'filename' in that
113            directory. The loop will terminate either when we successfully
114            exec or when we run out of path prefixes. */
115
116         prStart = PATH;
117         morePrefixes = 1;
118
119         while (morePrefixes) {
120
121             /* Locate end of prefix */
122
123             prEnd = strchr(prStart, ':');
124             if (prEnd == NULL)          /* Handle last prefix */
125                 prEnd = prStart + strlen(prStart);
126
127             /* Build complete pathname from path prefix and filename */
128
129             pathname = malloc(max(1, prEnd - prStart) + strlen(filename)
130                                 + 2);
131             pathname[0] = '\0';
132             if (prEnd == prStart)       /* Last prefix */
133                 strcat(pathname, ".");
134             else
135                 strncat(pathname, prStart, prEnd - prStart);
136             strcat(pathname, "/");
137             strcat(pathname, filename);
138
139             if (*prEnd == '\0')         /* No more prefixes */
140                 morePrefixes = 0;
141             else
142                 prStart = prEnd + 1;    /* Move to start of next prefix */
143
144             /* Try to exec pathname; execve() returns only if we failed. */
145
146             execve(pathname, argv, envp);
147             savedErrno = errno;         /* So we can return correct errno */
148             if (errno == EACCES)
149                 fndEACCES = 1;
150             else if (errno == ENOEXEC)
151                 execShScript(argc, argv, envp);
152
153             /* Avoid memory leaks, in case no execve() succeeds! */
154
155             free(pathname);
156         }
157
158         free(PATH);
159     }
160
161     /* If we get here, execve() failed; clean up and return -1, ensuring
162        that errno contains the value returned by (the last) execve() */
163
164     free(argv);
165     for (j = 0; envp[j] != NULL; j++)
166         free(envp[j]);
167     free(envp);
168
169     /* SUSv3 says that if any execve failed with EACCES then that
170        is the error that should be returned by exec[lv]p() */
171
172     errno = fndEACCES ? EACCES : savedErrno;
173     return -1;
174 }
