/*************************************************************************\
*                  Copyright (C) Michael Kerrisk, 2014.                   *
*                                                                         *
* This program is free software. You may use, modify, and redistribute it *
* under the terms of the GNU Affero General Public License as published   *
* by the Free Software Foundation, either version 3 or (at your option)   *
* any later version. This program is distributed without any warranty.    *
* See the file COPYING.agpl-v3 for details.                               *
\*************************************************************************/

/* Listing 24-6 */

 1 #include <signal.h>
 2 #include "curr_time.h"                  /* Declaration of currTime() */
 3 #include "tlpi_hdr.h"
 4
 5 #define SYNC_SIG SIGUSR1                /* Synchronization signal */
 6
 7 static void             /* Signal handler - does nothing but return */
 8 handler(int sig)
 9 {
10 }
11
12 int
13 main(int argc, char *argv[])
14 {
15     pid_t childPid;
16     sigset_t blockMask, origMask, emptyMask;
17     struct sigaction sa;
18
19     setbuf(stdout, NULL);               /* Disable buffering of stdout */
20
21     sigemptyset(&blockMask);
22     sigaddset(&blockMask, SYNC_SIG);    /* Block signal */
23     if (sigprocmask(SIG_BLOCK, &blockMask, &origMask) == -1)
24         errExit("sigprocmask");
25
26     sigemptyset(&sa.sa_mask);
27     sa.sa_flags = SA_RESTART;
28     sa.sa_handler = handler;
29     if (sigaction(SYNC_SIG, &sa, NULL) == -1)
30         errExit("sigaction");
31
32     switch (childPid = fork()) {
33     case -1:
34         errExit("fork");
35
36     case 0: /* Child */
37
38         /* Child does some required action here... */
39
40         printf("[%s %ld] Child started - doing some work\n",
41                 currTime("%T"), (long) getpid());
42         sleep(2);               /* Simulate time spent doing some work */
43
44         /* And then signals parent that it's done */
45
46         printf("[%s %ld] Child about to signal parent\n",
47                 currTime("%T"), (long) getpid());
48         if (kill(getppid(), SYNC_SIG) == -1)
49             errExit("kill");
50
51         /* Now child can do other things... */
52
53         _exit(EXIT_SUCCESS);
54
55     default: /* Parent */
56
57         /* Parent may do some work here, and then waits for child to
58            complete the required action */
59
60         printf("[%s %ld] Parent about to wait for signal\n",
61                 currTime("%T"), (long) getpid());
62         sigemptyset(&emptyMask);
63         if (sigsuspend(&emptyMask) == -1 && errno != EINTR)
64             errExit("sigsuspend");
65         printf("[%s %ld] Parent got signal\n", currTime("%T"), (long) getpid());
66
67         /* If required, return signal mask to its original state */
68
69         if (sigprocmask(SIG_SETMASK, &origMask, NULL) == -1)
70             errExit("sigprocmask");
71
72         /* Parent carries on to do other things... */
73
74         exit(EXIT_SUCCESS);
75     }
76 }
