/*************************************************************************\
*                  Copyright (C) Michael Kerrisk, 2014.                   *
*                                                                         *
* This program is free software. You may use, modify, and redistribute it *
* under the terms of the GNU Affero General Public License as published   *
* by the Free Software Foundation, either version 3 or (at your option)   *
* any later version. This program is distributed without any warranty.    *
* See the file COPYING.agpl-v3 for details.                               *
\*************************************************************************/

/* Listing 26-1 */

 1 #include <sys/wait.h>
 2 #include <time.h>
 3 #include "curr_time.h"              /* Declaration of currTime() */
 4 #include "tlpi_hdr.h"
 5
 6 int
 7 main(int argc, char *argv[])
 8 {
 9     int numDead;       /* Number of children so far waited for */
10     pid_t childPid;    /* PID of waited for child */
11     int j;
12
13     if (argc < 2 || strcmp(argv[1], "--help") == 0)
14         usageErr("%s sleep-time...\n", argv[0]);
15
16     setbuf(stdout, NULL);           /* Disable buffering of stdout */
17
18     for (j = 1; j < argc; j++) {    /* Create one child for each argument */
19         switch (fork()) {
20         case -1:
21             errExit("fork");
22
23         case 0:                     /* Child sleeps for a while then exits */
24             printf("[%s] child %d started with PID %ld, sleeping %s "
25                     "seconds\n", currTime("%T"), j, (long) getpid(),
26                     argv[j]);
27             sleep(getInt(argv[j], GN_NONNEG, "sleep-time"));
28             _exit(EXIT_SUCCESS);
29
30         default:                    /* Parent just continues around loop */
31             break;
32         }
33     }
34
35     numDead = 0;
36     for (;;) {                      /* Parent waits for each child to exit */
37         childPid = wait(NULL);
38         if (childPid == -1) {
39             if (errno == ECHILD) {
40                 printf("No more children - bye!\n");
41                 exit(EXIT_SUCCESS);
42             } else {                /* Some other (unexpected) error */
43                 errExit("wait");
44             }
45         }
46
47         numDead++;
48         printf("[%s] wait() returned child PID %ld (numDead=%d)\n",
49                 currTime("%T"), (long) childPid, numDead);
50     }
51 }
