/*************************************************************************\
*                  Copyright (C) Michael Kerrisk, 2014.                   *
*                                                                         *
* This program is free software. You may use, modify, and redistribute it *
* under the terms of the GNU Affero General Public License as published   *
* by the Free Software Foundation, either version 3 or (at your option)   *
* any later version. This program is distributed without any warranty.    *
* See the file COPYING.agpl-v3 for details.                               *
\*************************************************************************/

/* Listing 27-9 */

 1 #include <unistd.h>
 2 #include <signal.h>
 3 #include <sys/wait.h>
 4 #include <sys/types.h>
 5 #include <errno.h>
 6
 7 int
 8 system(const char *command)
 9 {
10     sigset_t blockMask, origMask;
11     struct sigaction saIgnore, saOrigQuit, saOrigInt, saDefault;
12     pid_t childPid;
13     int status, savedErrno;
14
15     if (command == NULL)                /* Is a shell available? */
16         return system(":") == 0;
17
18     sigemptyset(&blockMask);            /* Block SIGCHLD */
19     sigaddset(&blockMask, SIGCHLD);
20     sigprocmask(SIG_BLOCK, &blockMask, &origMask);
21
22     saIgnore.sa_handler = SIG_IGN;      /* Ignore SIGINT and SIGQUIT */
23     saIgnore.sa_flags = 0;
24     sigemptyset(&saIgnore.sa_mask);
25     sigaction(SIGINT, &saIgnore, &saOrigInt);
26     sigaction(SIGQUIT, &saIgnore, &saOrigQuit);
27
28     switch (childPid = fork()) {
29     case -1: /* fork() failed */
30         status = -1;
31         break;          /* Carry on to reset signal attributes */
32
33     case 0: /* Child: exec command */
34         saDefault.sa_handler = SIG_DFL;
35         saDefault.sa_flags = 0;
36         sigemptyset(&saDefault.sa_mask);
37
38         if (saOrigInt.sa_handler != SIG_IGN)
39             sigaction(SIGINT, &saDefault, NULL);
40         if (saOrigQuit.sa_handler != SIG_IGN)
41             sigaction(SIGQUIT, &saDefault, NULL);
42
43         sigprocmask(SIG_SETMASK, &origMask, NULL);
44
45         execl("/bin/sh", "sh", "-c", command, (char *) NULL);
46         _exit(127);                     /* We could not exec the shell */
47
48     default: /* Parent: wait for our child to terminate */
49         while (waitpid(childPid, &status, 0) == -1) {
50             if (errno != EINTR) {       /* Error other than EINTR */
51                 status = -1;
52                 break;                  /* So exit loop */
53             }
54         }
55         break;
56     }
57
58     /* Unblock SIGCHLD, restore dispositions of SIGINT and SIGQUIT */
59
60     savedErrno = errno;                 /* The following may change 'errno' */
61
62     sigprocmask(SIG_SETMASK, &origMask, NULL);
63     sigaction(SIGINT, &saOrigInt, NULL);
64     sigaction(SIGQUIT, &saOrigQuit, NULL);
65
66     errno = savedErrno;
67
68     return status;
69 }
