/*************************************************************************\
*                  Copyright (C) Michael Kerrisk, 2014.                   *
*                                                                         *
* This program is free software. You may use, modify, and redistribute it *
* under the terms of the GNU Affero General Public License as published   *
* by the Free Software Foundation, either version 3 or (at your option)   *
* any later version. This program is distributed without any warranty.    *
* See the file COPYING.agpl-v3 for details.                               *
\*************************************************************************/

/* Solution for Exercise 35-3 */

 1 #define _GNU_SOURCE
 2 #include <sched.h>
 3 #include <sys/resource.h>
 4 #include <sys/times.h>
 5 #include "tlpi_hdr.h"
 6
 7 #define CSEC_STEP 25            /* CPU centiseconds between messages */
 8
 9 static void
10 useCPU(char *msg)
11 {
12     struct tms tms;
13     int cpuCentisecs, prevStep, prevSec;
14
15     prevStep = 0;
16     prevSec = 0;
17     for (;;) {
18         if (times(&tms) == -1)
19             errExit("times");
20         cpuCentisecs = (tms.tms_utime + tms.tms_stime) * 100 /
21                         sysconf(_SC_CLK_TCK);
22
23         if (cpuCentisecs >= prevStep + CSEC_STEP) {
24             prevStep += CSEC_STEP;
25             printf("%s (PID %ld) cpu=%0.2f\n", msg, (long) getpid(),
26                     cpuCentisecs / 100.0);
27         }
28
29         if (cpuCentisecs > 300)         /* Terminate after 3 seconds */
30             break;
31
32         if (cpuCentisecs >= prevSec + 100) {    /* Yield once/second */
33             prevSec = cpuCentisecs;
34             sched_yield();
35         }
36     }
37 }
38
39 int
40 main(int argc, char *argv[])
41 {
42     struct rlimit rlim;
43     struct sched_param sp;
44     cpu_set_t set;
45
46     setbuf(stdout, NULL);               /* Disable buffering of stdout */
47
48     /* Confine all processes to a single CPU, so that the processes
49        won't run in parallel on multi-CPU systems. */
50
51     CPU_ZERO(&set);
52     CPU_SET(1, &set);
53
54     if (sched_setaffinity(getpid(), sizeof(set), &set) == -1)
55         errExit("sched_setaffinity");
56
57     /* Establish a CPU time limit. This demonstrates how we can
58        ensure that a runaway realtime process is terminated if we
59        make a programming error. The resource limit is inherited
60        by the child created using fork().
61
62        An alternative technique would be to make an alarm() call in each
63        process (since interval timers are not inherited across fork()). */
64
65     rlim.rlim_cur = rlim.rlim_max = 50;
66     if (setrlimit(RLIMIT_CPU, &rlim) == -1)
67         errExit("setrlimit");
68
69     /* Run the two processes in the lowest SCHED_FIFO priority */
70
71     sp.sched_priority = sched_get_priority_min(SCHED_FIFO);
72     if (sp.sched_priority == -1)
73         errExit("sched_get_priority_min");
74
75     if (sched_setscheduler(0, SCHED_FIFO, &sp) == -1)
76         errExit("sched_setscheduler");
77
78     switch (fork()) {
79     case -1:
80         errExit("fork");
81
82     case 0:
83         useCPU("child ");
84         exit(EXIT_SUCCESS);
85
86     default:
87         useCPU("parent");
88         exit(EXIT_SUCCESS);
89     }
90 }
