/*************************************************************************\
*                  Copyright (C) Michael Kerrisk, 2014.                   *
*                                                                         *
* This program is free software. You may use, modify, and redistribute it *
* under the terms of the GNU Affero General Public License as published   *
* by the Free Software Foundation, either version 3 or (at your option)   *
* any later version. This program is distributed without any warranty.    *
* See the file COPYING.agpl-v3 for details.                               *
\*************************************************************************/

/* Listing 36-3 */

 1 #include <sys/resource.h>
 2 #include "print_rlimit.h"               /* Declaration of printRlimit() */
 3 #include "tlpi_hdr.h"
 4
 5 int
 6 main(int argc, char *argv[])
 7 {
 8     struct rlimit rl;
 9     int j;
10     pid_t childPid;
11
12     if (argc < 2 || argc > 3 || strcmp(argv[1], "--help") == 0)
13         usageErr("%s soft-limit [hard-limit]\n", argv[0]);
14
15     printRlimit("Initial maximum process limits: ", RLIMIT_NPROC);
16
17     /* Set new process limits (hard == soft if not specified) */
18
19     rl.rlim_cur = (argv[1][0] == 'i') ? RLIM_INFINITY :
20                                 getInt(argv[1], 0, "soft-limit");
21     rl.rlim_max = (argc == 2) ? rl.rlim_cur :
22                 (argv[2][0] == 'i') ? RLIM_INFINITY :
23                                 getInt(argv[2], 0, "hard-limit");
24     if (setrlimit(RLIMIT_NPROC, &rl) == -1)
25         errExit("setrlimit");
26
27     printRlimit("New maximum process limits:     ", RLIMIT_NPROC);
28
29     /* Create as many children as possible */
30
31     for (j = 1; ; j++) {
32         switch (childPid = fork()) {
33         case -1: errExit("fork");
34
35         case 0: _exit(EXIT_SUCCESS);            /* Child */
36
37         default:        /* Parent: display message about each new child
38                            and let the resulting zombies accumulate */
39             printf("Child %d (PID=%ld) started\n", j, (long) childPid);
40             break;
41         }
42     }
43 }
