/*************************************************************************\
*                  Copyright (C) Michael Kerrisk, 2014.                   *
*                                                                         *
* This program is free software. You may use, modify, and redistribute it *
* under the terms of the GNU Affero General Public License as published   *
* by the Free Software Foundation, either version 3 or (at your option)   *
* any later version. This program is distributed without any warranty.    *
* See the file COPYING.agpl-v3 for details.                               *
\*************************************************************************/

/* Solution for Exercise 36-1 */

 1 #include <sys/wait.h>
 2 #include <signal.h>
 3 #include <time.h>
 4 #include <sys/resource.h>
 5 #include "tlpi_hdr.h"
 6
 7 #define NSECS 3         /* Amount of CPU to consume in child */
 8
 9 #define SIG SIGUSR1     /* Child uses this signal to tell parent
10                            that it is about to terminate */
11
12 static void
13 handler(int sig)
14 {
15     /* Do nothing: just interrupt sigsuspend() */
16 }
17
18 static void
19 printChildRusage(const char *msg)
20 {
21     struct rusage ru;
22
23     printf("%s", msg);
24     if (getrusage(RUSAGE_CHILDREN, &ru) == -1)
25         errExit("getrusage");
26     printf("user CPU=%.2f secs; system CPU=%.2f secs\n",
27             ru.ru_utime.tv_sec + ru.ru_utime.tv_usec / 1000000.0,
28             ru.ru_stime.tv_sec + ru.ru_stime.tv_usec / 1000000.0);
29 }
30
31 int
32 main(int argc, char *argv[])
33 {
34     clock_t start;
35     sigset_t mask;
36     struct sigaction sa;
37
38     setbuf(stdout, NULL);       /* Disable buffering of stdout */
39
40     sa.sa_handler = handler;
41     sa.sa_flags = 0;
42     sigemptyset(&sa.sa_mask);
43     if (sigaction(SIG, &sa, NULL) == -1)
44         errExit("sigaction");
45
46     /* Child informs parent of impending termination using a signal;
47        block that signal until the parent is ready to catch it. */
48
49     sigemptyset(&mask);
50     sigaddset(&mask, SIG);
51     if (sigprocmask(SIG_BLOCK, &mask, NULL) == -1)
52         errExit("sigprocmask");
53
54     switch (fork()) {
55     case -1:
56         errExit("fork");
57
58     case 0:             /* Child */
59         for (start = clock(); clock() - start < NSECS * CLOCKS_PER_SEC;)
60             continue;           /* Burn NSECS seconds of CPU time */
61         printf("Child terminating\n");
62
63         /* Tell parent we're nearly done */
64
65         if (kill(getppid(), SIG) == -1)
66             errExit("kill");
67         _exit(EXIT_SUCCESS);
68
69     default:    /* Parent */
70         sigemptyset(&mask);
71         sigsuspend(&mask);      /* Wait for signal from child */
72
73         sleep(2);               /* Allow child a bit more time to terminate */
74
75         printChildRusage("Before wait: ");
76         if (wait(NULL) == -1)
77             errExit("wait");
78         printChildRusage("After wait:  ");
79         exit(EXIT_SUCCESS);
80     }
81 }
