/*************************************************************************\
*                  Copyright (C) Michael Kerrisk, 2014.                   *
*                                                                         *
* This program is free software. You may use, modify, and redistribute it *
* under the terms of the GNU Affero General Public License as published   *
* by the Free Software Foundation, either version 3 or (at your option)   *
* any later version. This program is distributed without any warranty.    *
* See the file COPYING.agpl-v3 for details.                               *
\*************************************************************************/

/* Listing 53-1 */

 1 #include <semaphore.h>
 2 #include <sys/stat.h>
 3 #include <fcntl.h>
 4 #include "tlpi_hdr.h"
 5
 6 static void
 7 usageError(const char *progName)
 8 {
 9     fprintf(stderr, "Usage: %s [-cx] name [octal-perms [value]]\n", progName);
10     fprintf(stderr, "    -c   Create semaphore (O_CREAT)\n");
11     fprintf(stderr, "    -x   Create exclusively (O_EXCL)\n");
12     exit(EXIT_FAILURE);
13 }
14
15 int
16 main(int argc, char *argv[])
17 {
18     int flags, opt;
19     mode_t perms;
20     unsigned int value;
21     sem_t *sem;
22
23     flags = 0;
24     while ((opt = getopt(argc, argv, "cx")) != -1) {
25         switch (opt) {
26         case 'c':   flags |= O_CREAT;           break;
27         case 'x':   flags |= O_EXCL;            break;
28         default:    usageError(argv[0]);
29         }
30     }
31
32     if (optind >= argc)
33         usageError(argv[0]);
34
35     /* Default permissions are rw-------; default semaphore initialization
36        value is 0 */
37
38     perms = (argc <= optind + 1) ? (S_IRUSR | S_IWUSR) :
39                 getInt(argv[optind + 1], GN_BASE_8, "octal-perms");
40     value = (argc <= optind + 2) ? 0 : getInt(argv[optind + 2], 0, "value");
41
42     sem = sem_open(argv[optind], flags, perms, value);
43     if (sem == SEM_FAILED)
44         errExit("sem_open");
45
46     exit(EXIT_SUCCESS);
47 }
