/*************************************************************************\
*                  Copyright (C) Michael Kerrisk, 2014.                   *
*                                                                         *
* This program is free software. You may use, modify, and redistribute it *
* under the terms of the GNU Affero General Public License as published   *
* by the Free Software Foundation, either version 3 or (at your option)   *
* any later version. This program is distributed without any warranty.    *
* See the file COPYING.agpl-v3 for details.                               *
\*************************************************************************/

/* Listing 54-1 */

 1 #include <sys/stat.h>
 2 #include <fcntl.h>
 3 #include <sys/mman.h>
 4 #include "tlpi_hdr.h"
 5
 6 static void
 7 usageError(const char *progName)
 8 {
 9     fprintf(stderr, "Usage: %s [-cx] shm-name size [octal-perms]\n", progName);
10     fprintf(stderr, "    -c   Create shared memory (O_CREAT)\n");
11     fprintf(stderr, "    -x   Create exclusively (O_EXCL)\n");
12     exit(EXIT_FAILURE);
13 }
14
15 int
16 main(int argc, char *argv[])
17 {
18     int flags, opt, fd;
19     mode_t perms;
20     size_t size;
21     void *addr;
22
23     flags = O_RDWR;
24     while ((opt = getopt(argc, argv, "cx")) != -1) {
25         switch (opt) {
26         case 'c':   flags |= O_CREAT;           break;
27         case 'x':   flags |= O_EXCL;            break;
28         default:    usageError(argv[0]);
29         }
30     }
31
32     if (optind + 1 >= argc)
33         usageError(argv[0]);
34
35     size = getLong(argv[optind + 1], GN_ANY_BASE, "size");
36     perms = (argc <= optind + 2) ? (S_IRUSR | S_IWUSR) :
37                 getLong(argv[optind + 2], GN_BASE_8, "octal-perms");
38
39     /* Create shared memory object and set its size */
40
41     fd = shm_open(argv[optind], flags, perms);
42     if (fd == -1)
43         errExit("shm_open");
44
45     if (ftruncate(fd, size) == -1)
46         errExit("ftruncate");
47
48     /* Map shared memory object */
49
50     addr = mmap(NULL, size, PROT_READ | PROT_WRITE, MAP_SHARED, fd, 0);
51     if (addr == MAP_FAILED)
52         errExit("mmap");
53
54     exit(EXIT_SUCCESS);
55 }
