/*************************************************************************\
*                  Copyright (C) Michael Kerrisk, 2014.                   *
*                                                                         *
* This program is free software. You may use, modify, and redistribute it *
* under the terms of the GNU Affero General Public License as published   *
* by the Free Software Foundation, either version 3 or (at your option)   *
* any later version. This program is distributed without any warranty.    *
* See the file COPYING.agpl-v3 for details.                               *
\*************************************************************************/

/* Listing 64-4 */

 1 #include <fcntl.h>
 2 #include "pty_master_open.h"            /* Declares ptyMasterOpen() */
 3 #include "tlpi_hdr.h"
 4
 5 #define PTYM_PREFIX     "/dev/pty"
 6 #define PTYS_PREFIX     "/dev/tty"
 7 #define PTY_PREFIX_LEN  (sizeof(PTYM_PREFIX) - 1)
 8 #define PTY_NAME_LEN    (PTY_PREFIX_LEN + sizeof("XY"))
 9 #define X_RANGE         "pqrstuvwxyzabcde"
10 #define Y_RANGE         "0123456789abcdef"
11
12 int
13 ptyMasterOpen(char *slaveName, size_t snLen)
14 {
15     int masterFd, n;
16     char *x, *y;
17     char masterName[PTY_NAME_LEN];
18
19     if (PTY_NAME_LEN > snLen) {
20         errno = EOVERFLOW;
21         return -1;
22     }
23
24     memset(masterName, 0, PTY_NAME_LEN);
25     strncpy(masterName, PTYM_PREFIX, PTY_PREFIX_LEN);
26
27     for (x = X_RANGE; *x != '\0'; x++) {
28         masterName[PTY_PREFIX_LEN] = *x;
29
30         for (y = Y_RANGE; *y != '\0'; y++) {
31             masterName[PTY_PREFIX_LEN + 1] = *y;
32
33             masterFd = open(masterName, O_RDWR);
34
35             if (masterFd == -1) {
36                 if (errno == ENOENT)    /* No such file */
37                     return -1;          /* Probably no more pty devices */
38                 else                    /* Other error (e.g., pty busy) */
39                     continue;
40
41             } else {            /* Return slave name corresponding to master */
42                 n = snprintf(slaveName, snLen, "%s%c%c", PTYS_PREFIX, *x, *y);
43                 if (n >= snLen) {
44                     errno = EOVERFLOW;
45                     return -1;
46                 } else if (n == -1) {
47                     return -1;
48                 }
49
50                 return masterFd;
51             }
52         }
53     }
54
55     return -1;                  /* Tried all ptys without success */
56 }
