/*************************************************************************\
*                  Copyright (C) Michael Kerrisk, 2014.                   *
*                                                                         *
* This program is free software. You may use, modify, and redistribute it *
* under the terms of the GNU Affero General Public License as published   *
* by the Free Software Foundation, either version 3 or (at your option)   *
* any later version. This program is distributed without any warranty.    *
* See the file COPYING.agpl-v3 for details.                               *
\*************************************************************************/

/* Listing 64-3 */

 1 #include <sys/stat.h>
 2 #include <fcntl.h>
 3 #include <libgen.h>
 4 #include <termios.h>
 5 #include <sys/select.h>
 6 #include "pty_fork.h"           /* Declaration of ptyFork() */
 7 #include "tty_functions.h"      /* Declaration of ttySetRaw() */
 8 #include "tlpi_hdr.h"
 9
10 #define BUF_SIZE 256
11 #define MAX_SNAME 1000
12
13 struct termios ttyOrig;
14
15 static void             /* Reset terminal mode on program exit */
16 ttyReset(void)
17 {
18     if (tcsetattr(STDIN_FILENO, TCSANOW, &ttyOrig) == -1)
19         errExit("tcsetattr");
20 }
21
22 int
23 main(int argc, char *argv[])
24 {
25     char slaveName[MAX_SNAME];
26     char *shell;
27     int masterFd, scriptFd;
28     struct winsize ws;
29     fd_set inFds;
30     char buf[BUF_SIZE];
31     ssize_t numRead;
32     pid_t childPid;
33
34     if (tcgetattr(STDIN_FILENO, &ttyOrig) == -1)
35         errExit("tcgetattr");
36     if (ioctl(STDIN_FILENO, TIOCGWINSZ, &ws) < 0)
37         errExit("ioctl-TIOCGWINSZ");
38
39     childPid = ptyFork(&masterFd, slaveName, MAX_SNAME, &ttyOrig, &ws);
40     if (childPid == -1)
41         errExit("ptyFork");
42
43     if (childPid == 0) {        /* Child: execute a shell on pty slave */
44         shell = getenv("SHELL");
45         if (shell == NULL || *shell == '\0')
46             shell = "/bin/sh";
47
48         execlp(shell, shell, (char *) NULL);
49         errExit("execlp");      /* If we get here, something went wrong */
50     }
51
52     /* Parent: relay data between terminal and pty master */
53
54     scriptFd = open((argc > 1) ? argv[1] : "typescript",
55                         O_WRONLY | O_CREAT | O_TRUNC,
56                         S_IRUSR | S_IWUSR | S_IRGRP | S_IWGRP |
57                                 S_IROTH | S_IWOTH);
58     if (scriptFd == -1)
59         errExit("open typescript");
60
61     ttySetRaw(STDIN_FILENO, &ttyOrig);
62
63     if (atexit(ttyReset) != 0)
64         errExit("atexit");
65
66     for (;;) {
67         FD_ZERO(&inFds);
68         FD_SET(STDIN_FILENO, &inFds);
69         FD_SET(masterFd, &inFds);
70
71         if (select(masterFd + 1, &inFds, NULL, NULL, NULL) == -1)
72             errExit("select");
73
74         if (FD_ISSET(STDIN_FILENO, &inFds)) {   /* stdin --> pty */
75             numRead = read(STDIN_FILENO, buf, BUF_SIZE);
76             if (numRead <= 0)
77                 exit(EXIT_SUCCESS);
78
79             if (write(masterFd, buf, numRead) != numRead)
80                 fatal("partial/failed write (masterFd)");
81         }
82
83         if (FD_ISSET(masterFd, &inFds)) {      /* pty --> stdout+file */
84             numRead = read(masterFd, buf, BUF_SIZE);
85             if (numRead <= 0)
86                 exit(EXIT_SUCCESS);
87
88             if (write(STDOUT_FILENO, buf, numRead) != numRead)
89                 fatal("partial/failed write (STDOUT_FILENO)");
90             if (write(scriptFd, buf, numRead) != numRead)
91                 fatal("partial/failed write (scriptFd)");
92         }
93     }
94 }
