/*************************************************************************\
*                  Copyright (C) Michael Kerrisk, 2014.                   *
*                                                                         *
* This program is free software. You may use, modify, and redistribute it *
* under the terms of the GNU Affero General Public License as published   *
* by the Free Software Foundation, either version 3 or (at your option)   *
* any later version. This program is distributed without any warranty.    *
* See the file COPYING.agpl-v3 for details.                               *
\*************************************************************************/

/* Solution for Exercise 64-7 */

 1 #include <termios.h>
 2 #include <sys/select.h>
 3 #include "pty_fork.h"           /* Declaration of ptyFork() */
 4 #include "tty_functions.h"      /* Declaration of ttySetRaw() */
 5 #include "tlpi_hdr.h"
 6
 7 #define BUF_SIZE 256
 8 #define MAX_SNAME 1000
 9
10 struct termios ttyOrig;
11
12 static void             /* Reset terminal mode on program exit */
13 ttyReset(void)
14 {
15     if (tcsetattr(STDIN_FILENO, TCSANOW, &ttyOrig) == -1)
16         errExit("tcsetattr");
17 }
18
19 int
20 main(int argc, char *argv[])
21 {
22     char slaveName[MAX_SNAME];
23     int masterFd;
24     fd_set inFds;
25     char buf[BUF_SIZE];
26     ssize_t numRead;
27     struct winsize ws;
28
29     if (tcgetattr(STDIN_FILENO, &ttyOrig) == -1)
30         errExit("tcgetattr");
31     if (ioctl(STDIN_FILENO, TIOCGWINSZ, (char *) &ws) < 0)
32         errExit("TIOCGWINSZ error");
33
34     switch (ptyFork(&masterFd, slaveName, MAX_SNAME, &ttyOrig, &ws)) {
35     case -1:
36         errExit("ptyFork");
37
38     case 0:     /* Child executes command given in argv[1]... */
39         execvp(argv[1], &argv[1]);
40         errExit("execvp");
41
42     default:    /* Parent relays data between terminal and pty master */
43
44         ttySetRaw(STDIN_FILENO, &ttyOrig);
45         if (atexit(ttyReset) != 0)
46             errExit("atexit");
47
48         for (;;) {
49             FD_ZERO(&inFds);
50             FD_SET(STDIN_FILENO, &inFds);
51             FD_SET(masterFd, &inFds);
52             if (select(masterFd + 1, &inFds, NULL, NULL, NULL) == -1)
53                 errExit("select");
54
55             if (FD_ISSET(STDIN_FILENO, &inFds)) {
56                 numRead = read(STDIN_FILENO, buf, BUF_SIZE);
57                 if (numRead <= 0)
58                     exit(EXIT_SUCCESS);
59
60                 if (write(masterFd, buf, numRead) != numRead)
61                     fatal("partial/failed write (masterFd)");
62             }
63
64             if (FD_ISSET(masterFd, &inFds)) {
65                 numRead = read(masterFd, buf, BUF_SIZE);
66                 if (numRead <= 0)
67                     exit(EXIT_SUCCESS);
68
69                 if (write(STDOUT_FILENO, buf, numRead) != numRead)
70                     fatal("partial/failed write (STDOUT_FILENO)");
71             }
72         }
73     }
74 }
