/*************************************************************************\
*                  Copyright (C) Michael Kerrisk, 2014.                   *
*                                                                         *
* This program is free software. You may use, modify, and redistribute it *
* under the terms of the GNU Affero General Public License as published   *
* by the Free Software Foundation, either version 3 or (at your option)   *
* any later version. This program is distributed without any warranty.    *
* See the file COPYING.agpl-v3 for details.                               *
\*************************************************************************/

/* Supplementary program for Chapter 22 */

 1 #define _GNU_SOURCE     /* Get strsignal() declaration from <string.h> */
 2 #include <string.h>
 3 #include <signal.h>
 4 #include "tlpi_hdr.h"
 5
 6 static void
 7 sigfpeCatcher(int sig)
 8 {
 9     printf("Caught signal %d (%s)\n", sig, strsignal(sig));
10                                 /* UNSAFE (see Section 21.1.2) */
11     sleep(1);                   /* Slow down execution of handler */
12 }
13
14 int
15 main(int argc, char *argv[])
16 {
17     int x, y;
18     sigset_t blockSet, prevMask;
19     Boolean blocking;
20     struct sigaction sa;
21
22     /* If no command-line arguments specified, catch SIGFPE, else ignore it */
23
24     if (argc > 1 && strchr(argv[1], 'i') != NULL) {
25         printf("Ignoring SIGFPE\n");
26         if (signal(SIGFPE, SIG_IGN) == SIG_ERR)
27             errExit("signal");
28     } else {
29         printf("Catching SIGFPE\n");
30         sigemptyset(&sa.sa_mask);
31         sa.sa_flags = SA_RESTART;
32         sa.sa_handler = sigfpeCatcher;
33         if (sigaction(SIGFPE, &sa, NULL) == -1)
34             errExit("sigaction");
35     }
36
37     blocking = argc > 1 && strchr(argv[1], 'b') != NULL;
38     if (blocking) {
39         printf("Blocking SIGFPE\n");
40         sigemptyset(&blockSet);
41         sigaddset(&blockSet, SIGFPE);
42         if (sigprocmask(SIG_BLOCK, &blockSet, &prevMask) == -1)
43             errExit("sigprocmask");
44     }
45
46     printf("About to generate SIGFPE\n");
47     y = 0;
48     x = 1 / y;
49
50     if (blocking) {
51         printf("Sleeping before unblocking\n");
52         sleep(2);
53         printf("Unblocking SIGFPE\n");
54         if (sigprocmask(SIG_SETMASK, &prevMask, NULL) == -1)
55             errExit("sigprocmask");
56     }
57
58     printf("Shouldn't get here!\n");
59     exit(EXIT_FAILURE);
60 }
