/*************************************************************************\
*                  Copyright (C) Michael Kerrisk, 2014.                   *
*                                                                         *
* This program is free software. You may use, modify, and redistribute it *
* under the terms of the GNU Affero General Public License as published   *
* by the Free Software Foundation, either version 3 or (at your option)   *
* any later version. This program is distributed without any warranty.    *
* See the file COPYING.agpl-v3 for details.                               *
\*************************************************************************/

/* Solution for Exercise 20-2 */

 1 #define _GNU_SOURCE     /* Get strsignal() declaration from <string.h> */
 2 #include <string.h>
 3 #include <signal.h>
 4 #include "signal_functions.h"   /* Declaration of printSigset() */
 5 #include "tlpi_hdr.h"
 6
 7 static void
 8 handler(int sig)
 9 {
10     /* UNSAFE: This handler uses non-async-signal-safe functions
11        (printf(), strsignal(), fflush(); see Section 21.1.2) */
12
13     printf("Caught signal %d (%s)\n", sig, strsignal(sig));
14     fflush(NULL);
15 }
16
17 int
18 main(int argc, char *argv[])
19 {
20     sigset_t pending, blocked;
21     const int numSecs = 5;
22     struct sigaction sa;
23
24     /* Set up a handler for SIGINT */
25
26     printf("Setting up handler for SIGINT\n");
27     sigemptyset(&sa.sa_mask);
28     sa.sa_flags = 0;
29     sa.sa_handler = handler;
30     if (sigaction(SIGINT, &sa, NULL) == -1)
31         errExit("sigaction");
32
33     /* Block SIGINT for a while */
34
35     sigemptyset(&blocked);
36     sigaddset(&blocked, SIGINT);
37     if (sigprocmask(SIG_SETMASK, &blocked, NULL) == -1)
38         errExit("sigprocmask");
39
40     printf("BLOCKING SIGINT for %d seconds\n", numSecs);
41     sleep(numSecs);
42
43     /* Display mask of pending signals */
44
45     if (sigpending(&pending) == -1)
46         errExit("sigpending");
47     printf("PENDING signals are: \n");
48     printSigset(stdout, "\t\t", &pending);
49
50     /* Now ignore SIGINT */
51
52     sleep(2);
53     printf("Ignoring SIGINT\n");
54     if (signal(SIGINT, SIG_IGN) == SIG_ERR)
55         errExit("signal");
56
57     /* Redisplay mask of pending signals */
58
59     if (sigpending(&pending) == -1)
60         errExit("sigpending");
61     if (sigismember(&pending, SIGINT)) {
62         printf("SIGINT is now pending\n");
63     } else {
64         printf("PENDING signals are: \n");
65         printSigset(stdout, "\t\t", &pending);
66     }
67     sleep(2);
68
69     /* Reestablish SIGINT handler */
70
71     printf("Reestablishing handler for SIGINT\n");
72     sigemptyset(&sa.sa_mask);
73     sa.sa_flags = 0;
74     sa.sa_handler = handler;
75     if (sigaction(SIGINT, &sa, NULL) == -1)
76         errExit("sigaction");
77
78     sleep(2);
79
80     /* And unblock SIGINT */
81
82     printf("UNBLOCKING SIGINT\n");
83     sigemptyset(&blocked);
84     if (sigprocmask(SIG_SETMASK, &blocked, NULL) == -1)
85         errExit("sigprocmask");
86
87     exit(EXIT_SUCCESS);
88 }
