/*************************************************************************\
*                  Copyright (C) Michael Kerrisk, 2014.                   *
*                                                                         *
* This program is free software. You may use, modify, and redistribute it *
* under the terms of the GNU Affero General Public License as published   *
* by the Free Software Foundation, either version 3 or (at your option)   *
* any later version. This program is distributed without any warranty.    *
* See the file COPYING.agpl-v3 for details.                               *
\*************************************************************************/

/* Supplementary program for Chapter 22 */

 1 #include <signal.h>
 2 #include "tlpi_hdr.h"
 3
 4 static void
 5 handler(int sig)
 6 {
 7 }
 8
 9 #define TESTSIG SIGUSR1
10
11 int
12 main(int argc, char *argv[])
13 {
14     int numSigs, scnt;
15     pid_t childPid;
16     sigset_t blockedMask, emptyMask;
17     struct sigaction sa;
18
19     if (argc != 2 || strcmp(argv[1], "--help") == 0)
20         usageErr("%s num-sigs\n", argv[0]);
21
22     numSigs = getInt(argv[1], GN_GT_0, "num-sigs");
23
24     sigemptyset(&sa.sa_mask);
25     sa.sa_flags = 0;
26     sa.sa_handler = handler;
27     if (sigaction(TESTSIG, &sa, NULL) == -1)
28         errExit("sigaction");
29
30     /* Block the signal before fork(), so that the child doesn't manage
31        to send it to the parent before the parent is ready to catch it */
32
33     sigemptyset(&blockedMask);
34     sigaddset(&blockedMask, TESTSIG);
35     if (sigprocmask(SIG_SETMASK, &blockedMask, NULL) == -1)
36         errExit("sigprocmask");
37
38     sigemptyset(&emptyMask);
39
40     switch (childPid = fork()) {
41     case -1: errExit("fork");
42
43     case 0:     /* child */
44         for (scnt = 0; scnt < numSigs; scnt++) {
45             if (kill(getppid(), TESTSIG) == -1)
46                 errExit("kill");
47             if (sigsuspend(&emptyMask) == -1 && errno != EINTR)
48                     errExit("sigsuspend");
49         }
50         exit(EXIT_SUCCESS);
51
52     default: /* parent */
53         for (scnt = 0; scnt < numSigs; scnt++) {
54             if (sigsuspend(&emptyMask) == -1 && errno != EINTR)
55                     errExit("sigsuspend");
56             if (kill(childPid, TESTSIG) == -1)
57                 errExit("kill");
58         }
59         exit(EXIT_SUCCESS);
60     }
61 }
