/*************************************************************************\
*                  Copyright (C) Michael Kerrisk, 2014.                   *
*                                                                         *
* This program is free software. You may use, modify, and redistribute it *
* under the terms of the GNU Affero General Public License as published   *
* by the Free Software Foundation, either version 3 or (at your option)   *
* any later version. This program is distributed without any warranty.    *
* See the file COPYING.agpl-v3 for details.                               *
\*************************************************************************/

/* Listing 21-2 */

 1 #define _GNU_SOURCE     /* Get strsignal() declaration from <string.h> */
 2 #include <string.h>
 3 #include <setjmp.h>
 4 #include <signal.h>
 5 #include "signal_functions.h"           /* Declaration of printSigMask() */
 6 #include "tlpi_hdr.h"
 7
 8 static volatile sig_atomic_t canJump = 0;
 9                         /* Set to 1 once "env" buffer has been
10                            initialized by [sig]setjmp() */
11 #ifdef USE_SIGSETJMP
12 static sigjmp_buf senv;
13 #else
14 static jmp_buf env;
15 #endif
16
17 static void
18 handler(int sig)
19 {
20     /* UNSAFE: This handler uses non-async-signal-safe functions
21        (printf(), strsignal(), printSigMask(); see Section 21.1.2) */
22
23     printf("Received signal %d (%s), signal mask is:\n", sig,
24             strsignal(sig));
25     printSigMask(stdout, NULL);
26
27     if (!canJump) {
28         printf("'env' buffer not yet set, doing a simple return\n");
29         return;
30     }
31
32 #ifdef USE_SIGSETJMP
33     siglongjmp(senv, 1);
34 #else
35     longjmp(env, 1);
36 #endif
37 }
38
39 int
40 main(int argc, char *argv[])
41 {
42     struct sigaction sa;
43
44     printSigMask(stdout, "Signal mask at startup:\n");
45
46     sigemptyset(&sa.sa_mask);
47     sa.sa_flags = 0;
48     sa.sa_handler = handler;
49     if (sigaction(SIGINT, &sa, NULL) == -1)
50         errExit("sigaction");
51
52 #ifdef USE_SIGSETJMP
53     printf("Calling sigsetjmp()\n");
54     if (sigsetjmp(senv, 1) == 0)
55 #else
56     printf("Calling setjmp()\n");
57     if (setjmp(env) == 0)
58 #endif
59         canJump = 1;                    /* Executed after [sig]setjmp() */
60
61     else                                /* Executed after [sig]longjmp() */
62         printSigMask(stdout, "After jump from handler, signal mask is:\n" );
63
64     for (;;)                            /* Wait for signals until killed */
65         pause();
66 }
