 1 /*************************************************************************\
 2 *                  Copyright (C) Michael Kerrisk, 2014.                   *
 3 *                                                                         *
 4 * This program is free software. You may use, modify, and redistribute it *
 5 * under the terms of the GNU Lesser General Public License as published   *
 6 * by the Free Software Foundation, either version 3 or (at your option)   *
 7 * any later version. This program is distributed without any warranty.    *
 8 * See the files COPYING.lgpl-v3 and COPYING.gpl-v3 for details.           *
 9 \*************************************************************************/
10
11 /* Listing 20-4 */
12
13  1 #define _GNU_SOURCE
14  2 #include <string.h>
15  3 #include <signal.h>
16  4 #include "signal_functions.h"           /* Declares functions defined here */
17  5 #include "tlpi_hdr.h"
18  6
19  7 /* NOTE: All of the following functions employ fprintf(), which
20  8    is not async-signal-safe (see Section 21.1.2). As such, these
21  9    functions are also not async-signal-safe (i.e., beware of
22 10    indiscriminately calling them from signal handlers). */
23 11
24 12 void                    /* Print list of signals within a signal set */
25 13 printSigset(FILE *of, const char *prefix, const sigset_t *sigset)
26 14 {
27 15     int sig, cnt;
28 16
29 17     cnt = 0;
30 18     for (sig = 1; sig < NSIG; sig++) {
31 19         if (sigismember(sigset, sig)) {
32 20             cnt++;
33 21             fprintf(of, "%s%d (%s)\n", prefix, sig, strsignal(sig));
34 22         }
35 23     }
36 24
37 25     if (cnt == 0)
38 26         fprintf(of, "%s<empty signal set>\n", prefix);
39 27 }
40 28
41 29 int                     /* Print mask of blocked signals for this process */
42 30 printSigMask(FILE *of, const char *msg)
43 31 {
44 32     sigset_t currMask;
45 33
46 34     if (msg != NULL)
47 35         fprintf(of, "%s", msg);
48 36
49 37     if (sigprocmask(SIG_BLOCK, NULL, &currMask) == -1)
50 38         return -1;
51 39
52 40     printSigset(of, "\t\t", &currMask);
53 41
54 42     return 0;
55 43 }
56 44
57 45 int                     /* Print signals currently pending for this process */
58 46 printPendingSigs(FILE *of, const char *msg)
59 47 {
60 48     sigset_t pendingSigs;
61 49
62 50     if (msg != NULL)
63 51         fprintf(of, "%s", msg);
64 52
65 53     if (sigpending(&pendingSigs) == -1)
66 54         return -1;
67 55
68 56     printSigset(of, "\t\t", &pendingSigs);
69 57
70 58     return 0;
71 59 }
