/*************************************************************************\
*                  Copyright (C) Michael Kerrisk, 2014.                   *
*                                                                         *
* This program is free software. You may use, modify, and redistribute it *
* under the terms of the GNU Affero General Public License as published   *
* by the Free Software Foundation, either version 3 or (at your option)   *
* any later version. This program is distributed without any warranty.    *
* See the file COPYING.agpl-v3 for details.                               *
\*************************************************************************/

/* Listing 22-6 */

 1 #define _GNU_SOURCE
 2 #include <string.h>
 3 #include <signal.h>
 4 #include <time.h>
 5 #include "tlpi_hdr.h"
 6
 7 int
 8 main(int argc, char *argv[])
 9 {
10     int sig;
11     siginfo_t si;
12     sigset_t allSigs;
13
14     if (argc > 1 && strcmp(argv[1], "--help") == 0)
15         usageErr("%s [delay-secs]\n", argv[0]);
16
17     printf("%s: PID is %ld\n", argv[0], (long) getpid());
18
19     /* Block all signals (except SIGKILL and SIGSTOP) */
20
21     sigfillset(&allSigs);
22     if (sigprocmask(SIG_SETMASK, &allSigs, NULL) == -1)
23         errExit("sigprocmask");
24     printf("%s: signals blocked\n", argv[0]);
25
26     if (argc > 1) {             /* Delay so that signals can be sent to us */
27         printf("%s: about to delay %s seconds\n", argv[0], argv[1]);
28         sleep(getInt(argv[1], GN_GT_0, "delay-secs"));
29         printf("%s: finished delay\n", argv[0]);
30     }
31
32     for (;;) {                  /* Fetch signals until SIGINT (^C) or SIGTERM */
33         sig = sigwaitinfo(&allSigs, &si);
34         if (sig == -1)
35             errExit("sigwaitinfo");
36
37         if (sig == SIGINT || sig == SIGTERM)
38             exit(EXIT_SUCCESS);
39
40         printf("got signal: %d (%s)\n", sig, strsignal(sig));
41         printf("    si_signo=%d, si_code=%d (%s), si_value=%d\n",
42                 si.si_signo, si.si_code,
43                 (si.si_code == SI_USER) ? "SI_USER" :
44                     (si.si_code == SI_QUEUE) ? "SI_QUEUE" : "other",
45                 si.si_value.sival_int);
46         printf("    si_pid=%ld, si_uid=%ld\n",
47                 (long) si.si_pid, (long) si.si_uid);
48     }
49 }
