/*************************************************************************\
*                  Copyright (C) Michael Kerrisk, 2014.                   *
*                                                                         *
* This program is free software. You may use, modify, and redistribute it *
* under the terms of the GNU Affero General Public License as published   *
* by the Free Software Foundation, either version 3 or (at your option)   *
* any later version. This program is distributed without any warranty.    *
* See the file COPYING.agpl-v3 for details.                               *
\*************************************************************************/

/* Solution for Exercise 60-2 */

 1 #include <syslog.h>
 2 #include <signal.h>
 3 #include <sys/socket.h>
 4 #include <sys/wait.h>
 5 #include "become_daemon.h"
 6 #include "inet_sockets.h"       /* Declares our socket functions */
 7 #include "tlpi_hdr.h"
 8
 9 #define SERVICE "echo"          /* Name of TCP service */
10 #define BUF_SIZE 4096
11
12 static void             /* SIGCHLD handler to reap dead child processes */
13 grimReaper(int sig)
14 {
15     int savedErrno;             /* Save 'errno' in case changed here */
16
17     savedErrno = errno;
18     while (waitpid(-1, NULL, WNOHANG) > 0)
19         continue;
20     errno = savedErrno;
21 }
22
23 static void             /* Handle client: copy socket input back to socket */
24 handleRequest(int cfd)
25 {
26     ssize_t numRead;
27     char buf[BUF_SIZE];
28
29     while ((numRead = read(cfd, buf, BUF_SIZE)) > 0) {
30         if (write(cfd, buf, numRead) != numRead) {
31             syslog(LOG_ERR, "write() failed: %s", strerror(errno));
32             exit(EXIT_FAILURE);
33         }
34     }
35
36     if (numRead == -1) {
37         syslog(LOG_ERR, "Error from read(): %s", strerror(errno));
38         exit(EXIT_FAILURE);
39     }
40 }
41
42 int
43 main(int argc, char *argv[])
44 {
45     int lfd, cfd;               /* Listening and connected sockets */
46     struct sigaction sa;
47
48     /* The "-i" option means we were invoked from inetd(8), so that
49        all we need to do is handle the connection on STDIN_FILENO */
50
51     if (argc > 1 && strcmp(argv[1], "-i") == 0) {
52         handleRequest(STDIN_FILENO);
53         exit(EXIT_SUCCESS);
54     }
55
56     if (becomeDaemon(0) == -1)
57         errExit("becomeDaemon");
58
59     sigemptyset(&sa.sa_mask);
60     sa.sa_flags = SA_RESTART;
61     sa.sa_handler = grimReaper;
62     if (sigaction(SIGCHLD, &sa, NULL) == -1) {
63         syslog(LOG_ERR, "Error from sigaction(): %s", strerror(errno));
64         exit(EXIT_FAILURE);
65     }
66
67     lfd = inetListen(SERVICE, 10, NULL);
68     if (lfd == -1) {
69         syslog(LOG_ERR, "Could not create server socket (%s)", strerror(errno));
70         exit(EXIT_FAILURE);
71     }
72
73     for (;;) {
74         cfd = accept(lfd, NULL, 0);     /* Wait for connection */
75         if (cfd == -1) {
76             syslog(LOG_ERR, "Failure in accept(): %s",
77                     strerror(errno));
78             continue;           /* Try next */
79         }
80
81         switch (fork()) {       /* Create child for each client */
82         case -1:
83             syslog(LOG_ERR, "Can't create child (%s)",
84                     strerror(errno));
85             close(cfd);         /* Give up on this client */
86             break;              /* May be temporary; try next client */
87
88         case 0:                 /* Child */
89             close(lfd);         /* Don't need copy of listening socket */
90             handleRequest(cfd);
91             _exit(EXIT_SUCCESS);
92
93         default:                /* Parent */
94             close(cfd);         /* Don't need copy of connected socket */
95             break;              /* Loop to accept next connection */
96         }
97     }
98 }
