/*************************************************************************\
*                  Copyright (C) Michael Kerrisk, 2014.                   *
*                                                                         *
* This program is free software. You may use, modify, and redistribute it *
* under the terms of the GNU Affero General Public License as published   *
* by the Free Software Foundation, either version 3 or (at your option)   *
* any later version. This program is distributed without any warranty.    *
* See the file COPYING.agpl-v3 for details.                               *
\*************************************************************************/

/* Listing 59-7 */

 1 #include <netdb.h>
 2 #include "is_seqnum.h"
 3
 4 int
 5 main(int argc, char *argv[])
 6 {
 7     char *reqLenStr;                    /* Requested length of sequence */
 8     char seqNumStr[INT_LEN];            /* Start of granted sequence */
 9     int cfd;
10     ssize_t numRead;
11     struct addrinfo hints;
12     struct addrinfo *result, *rp;
13
14     if (argc < 2 || strcmp(argv[1], "--help") == 0)
15         usageErr("%s server-host [sequence-len]\n", argv[0]);
16
17     /* Call getaddrinfo() to obtain a list of addresses that
18        we can try connecting to */
19
20     memset(&hints, 0, sizeof(struct addrinfo));
21     hints.ai_canonname = NULL;
22     hints.ai_addr = NULL;
23     hints.ai_next = NULL;
24     hints.ai_family = AF_UNSPEC;                /* Allows IPv4 or IPv6 */
25     hints.ai_socktype = SOCK_STREAM;
26     hints.ai_flags = AI_NUMERICSERV;
27
28     if (getaddrinfo(argv[1], PORT_NUM, &hints, &result) != 0)
29         errExit("getaddrinfo");
30
31     /* Walk through returned list until we find an address structure
32        that can be used to successfully connect a socket */
33
34     for (rp = result; rp != NULL; rp = rp->ai_next) {
35         cfd = socket(rp->ai_family, rp->ai_socktype, rp->ai_protocol);
36         if (cfd == -1)
37             continue;                           /* On error, try next address */
38
39         if (connect(cfd, rp->ai_addr, rp->ai_addrlen) != -1)
40             break;                              /* Success */
41
42         /* Connect failed: close this socket and try next address */
43
44         close(cfd);
45     }
46
47     if (rp == NULL)
48         fatal("Could not connect socket to any address");
49
50     freeaddrinfo(result);
51
52     /* Send requested sequence length, with terminating newline */
53
54     reqLenStr = (argc > 2) ? argv[2] : "1";
55     if (write(cfd, reqLenStr, strlen(reqLenStr)) !=  strlen(reqLenStr))
56         fatal("Partial/failed write (reqLenStr)");
57     if (write(cfd, "\n", 1) != 1)
58         fatal("Partial/failed write (newline)");
59
60     /* Read and display sequence number returned by server */
61
62     numRead = readLine(cfd, seqNumStr, INT_LEN);
63     if (numRead == -1)
64         errExit("readLine");
65     if (numRead == 0)
66         fatal("Unexpected EOF from server");
67
68     printf("Sequence number: %s", seqNumStr);   /* Includes '\n' */
69
70     exit(EXIT_SUCCESS);                         /* Closes 'cfd' */
71 }
