 1 /*************************************************************************\
 2 *                  Copyright (C) Michael Kerrisk, 2014.                   *
 3 *                                                                         *
 4 * This program is free software. You may use, modify, and redistribute it *
 5 * under the terms of the GNU Lesser General Public License as published   *
 6 * by the Free Software Foundation, either version 3 or (at your option)   *
 7 * any later version. This program is distributed without any warranty.    *
 8 * See the files COPYING.lgpl-v3 and COPYING.gpl-v3 for details.           *
 9 \*************************************************************************/
10
11 /* Listing 61-1 */
12
13  1 #include <unistd.h>
14  2 #include <errno.h>
15  3 #include "rdwrn.h"                      /* Declares readn() and writen() */
16  4
17  5 ssize_t
18  6 readn(int fd, void *buffer, size_t n)
19  7 {
20  8     ssize_t numRead;                    /* # of bytes fetched by last read() */
21  9     size_t totRead;                     /* Total # of bytes read so far */
22 10     char *buf;
23 11
24 12     buf = buffer;                       /* No pointer arithmetic on "void *" */
25 13     for (totRead = 0; totRead < n; ) {
26 14         numRead = read(fd, buf, n - totRead);
27 15
28 16         if (numRead == 0)               /* EOF */
29 17             return totRead;             /* May be 0 if this is first read() */
30 18         if (numRead == -1) {
31 19             if (errno == EINTR)
32 20                 continue;               /* Interrupted --> restart read() */
33 21             else
34 22                 return -1;              /* Some other error */
35 23         }
36 24         totRead += numRead;
37 25         buf += numRead;
38 26     }
39 27     return totRead;                     /* Must be 'n' bytes if we get here */
40 28 }
41 29
42 30 ssize_t
43 31 writen(int fd, const void *buffer, size_t n)
44 32 {
45 33     ssize_t numWritten;                 /* # of bytes written by last write() */
46 34     size_t totWritten;                  /* Total # of bytes written so far */
47 35     const char *buf;
48 36
49 37     buf = buffer;                       /* No pointer arithmetic on "void *" */
50 38     for (totWritten = 0; totWritten < n; ) {
51 39         numWritten = write(fd, buf, n - totWritten);
52 40
53 41         if (numWritten <= 0) {
54 42             if (numWritten == -1 && errno == EINTR)
55 43                 continue;               /* Interrupted --> restart write() */
56 44             else
57 45                 return -1;              /* Some other error */
58 46         }
59 47         totWritten += numWritten;
60 48         buf += numWritten;
61 49     }
62 50     return totWritten;                  /* Must be 'n' bytes if we get here */
63 51 }
