/*************************************************************************\
*                  Copyright (C) Michael Kerrisk, 2014.                   *
*                                                                         *
* This program is free software. You may use, modify, and redistribute it *
* under the terms of the GNU Affero General Public License as published   *
* by the Free Software Foundation, either version 3 or (at your option)   *
* any later version. This program is distributed without any warranty.    *
* See the file COPYING.agpl-v3 for details.                               *
\*************************************************************************/

/* Supplementary program for Chapter 61 */

  1 #include "scm_rights.h"
  2
  3 #define BUF_SIZE 100
  4
  5 int
  6 main(int argc, char *argv[])
  7 {
  8     struct msghdr msgh;
  9     struct iovec iov;
 10     int data, lfd, sfd, fd, opt;
 11     ssize_t nr;
 12     Boolean useDatagramSocket;
 13     union {
 14         struct cmsghdr cmh;
 15         char   control[CMSG_SPACE(sizeof(int))];
 16                         /* Space large enough to hold an 'int' */
 17     } control_un;
 18     struct cmsghdr *cmhp;
 19
 20     /* Parse command-line arguments */
 21
 22     useDatagramSocket = FALSE;
 23
 24     while ((opt = getopt(argc, argv, "d")) != -1) {
 25         switch (opt) {
 26         case 'd':
 27             useDatagramSocket = TRUE;
 28             break;
 29
 30         default:
 31             usageErr("%s [-d]\n"
 32                      "        -d    use datagram socket\n", argv[0]);
 33         }
 34     }
 35
 36     /* Create socket bound to well-known address */
 37
 38     if (remove(SOCK_PATH) == -1 && errno != ENOENT)
 39         errExit("remove-%s", SOCK_PATH);
 40
 41     if (useDatagramSocket) {
 42         fprintf(stderr, "Receiving via datagram socket\n");
 43         sfd = unixBind(SOCK_PATH, SOCK_DGRAM);
 44         if (sfd == -1)
 45             errExit("unixBind");
 46
 47     } else {
 48         fprintf(stderr, "Receiving via stream socket\n");
 49         lfd = unixListen(SOCK_PATH, 5);
 50         if (lfd == -1)
 51             errExit("unixListen");
 52
 53         sfd = accept(lfd, NULL, 0);
 54         if (sfd == -1)
 55             errExit("accept");
 56     }
 57
 58     /* Set 'control_un' to describe ancillary data that we want to receive */
 59
 60     control_un.cmh.cmsg_len = CMSG_LEN(sizeof(int));
 61     control_un.cmh.cmsg_level = SOL_SOCKET;
 62     control_un.cmh.cmsg_type = SCM_RIGHTS;
 63
 64     /* Set 'msgh' fields to describe 'control_un' */
 65
 66     msgh.msg_control = control_un.control;
 67     msgh.msg_controllen = sizeof(control_un.control);
 68
 69     /* Set fields of 'msgh' to point to buffer used to receive (real)
 70        data read by recvmsg() */
 71
 72     msgh.msg_iov = &iov;
 73     msgh.msg_iovlen = 1;
 74     iov.iov_base = &data;
 75     iov.iov_len = sizeof(int);
 76
 77     msgh.msg_name = NULL;               /* We don't need address of peer */
 78     msgh.msg_namelen = 0;
 79
 80     /* Receive real plus ancillary data */
 81
 82     nr = recvmsg(sfd, &msgh, 0);
 83     if (nr == -1)
 84         errExit("recvmsg");
 85     fprintf(stderr, "recvmsg() returned %ld\n", (long) nr);
 86
 87     if (nr > 0)
 88         fprintf(stderr, "Received data = %d\n", data);
 89
 90     /* Get the received file descriptor (which is typically a different
 91        file descriptor number than was used in the sending process) */
 92
 93     cmhp = CMSG_FIRSTHDR(&msgh);
 94     if (cmhp == NULL || cmhp->cmsg_len != CMSG_LEN(sizeof(int)))
 95         fatal("bad cmsg header / message length");
 96     if (cmhp->cmsg_level != SOL_SOCKET)
 97         fatal("cmsg_level != SOL_SOCKET");
 98     if (cmhp->cmsg_type != SCM_RIGHTS)
 99         fatal("cmsg_type != SCM_RIGHTS");
100
101     fd = *((int *) CMSG_DATA(cmhp));
102     fprintf(stderr, "Received fd=%d\n", fd);
103
104     /* Having obtained the file descriptor, read the file's contents and
105        print them on standard output */
106
107     for (;;) {
108         char buf[BUF_SIZE];
109         ssize_t numRead;
110
111         numRead = read(fd, buf, BUF_SIZE);
112         if (numRead == -1)
113             errExit("read");
114
115         if (numRead == 0)
116             break;
117
118         write(STDOUT_FILENO, buf, numRead);
119     }
120
121     exit(EXIT_SUCCESS);
122 }
