/*************************************************************************\
*                  Copyright (C) Michael Kerrisk, 2014.                   *
*                                                                         *
* This program is free software. You may use, modify, and redistribute it *
* under the terms of the GNU Affero General Public License as published   *
* by the Free Software Foundation, either version 3 or (at your option)   *
* any later version. This program is distributed without any warranty.    *
* See the file COPYING.agpl-v3 for details.                               *
\*************************************************************************/

/* Supplementary program for Chapter 61 */

 1 #include "scm_rights.h"
 2
 3 int
 4 main(int argc, char *argv[])
 5 {
 6     struct msghdr msgh;
 7     struct iovec iov;
 8     int data, sfd, opt, fd;
 9     ssize_t ns;
10     Boolean useDatagramSocket;
11     union {
12         struct cmsghdr cmh;
13         char   control[CMSG_SPACE(sizeof(int))];
14                         /* Space large enough to hold an 'int' */
15     } control_un;
16     struct cmsghdr *cmhp;
17
18     /* Parse command-line arguments */
19
20     useDatagramSocket = FALSE;
21
22     while ((opt = getopt(argc, argv, "d")) != -1) {
23         switch (opt) {
24         case 'd':
25             useDatagramSocket = TRUE;
26             break;
27
28         default:
29             usageErr("%s [-d] file\n"
30                      "        -d    use datagram socket\n", argv[0]);
31         }
32     }
33
34     if (argc != optind + 1)
35         usageErr("%s [-d] file\n", argv[0]);
36
37     /* Open the file named on the command line */
38
39     fd = open(argv[optind], O_RDONLY);
40     if (fd == -1)
41         errExit("open");
42
43     /* On Linux, we must transmit at least 1 byte of real data in
44        order to send ancillary data */
45
46     msgh.msg_iov = &iov;
47     msgh.msg_iovlen = 1;
48     iov.iov_base = &data;
49     iov.iov_len = sizeof(int);
50     data = 12345;
51
52     /* We don't need to specify destination address, because we use
53        connect() below */
54
55     msgh.msg_name = NULL;
56     msgh.msg_namelen = 0;
57
58     msgh.msg_control = control_un.control;
59     msgh.msg_controllen = sizeof(control_un.control);
60
61     fprintf(stderr, "Sending fd %d\n", fd);
62
63     /* Set message header to describe ancillary data that we want to send */
64
65     cmhp = CMSG_FIRSTHDR(&msgh);
66     cmhp->cmsg_len = CMSG_LEN(sizeof(int));
67     cmhp->cmsg_level = SOL_SOCKET;
68     cmhp->cmsg_type = SCM_RIGHTS;
69     *((int *) CMSG_DATA(cmhp)) = fd;
70
71     /*
72     We could rewrite the preceding lines as:
73
74     control_un.cmh.cmsg_len = CMSG_LEN(sizeof(int));
75     control_un.cmh.cmsg_level = SOL_SOCKET;
76     control_un.cmh.cmsg_type = SCM_RIGHTS;
77     *((int *) CMSG_DATA(CMSG_FIRSTHDR(&msgh))) = fd;
78     */
79
80     /* Do the actual send */
81
82     sfd = unixConnect(SOCK_PATH, useDatagramSocket ? SOCK_DGRAM : SOCK_STREAM);
83     if (sfd == -1)
84         errExit("unixConnect");
85
86     ns = sendmsg(sfd, &msgh, 0);
87     if (ns == -1)
88         errExit("sendmsg");
89
90     fprintf(stderr, "sendmsg() returned %ld\n", (long) ns);
91     exit(EXIT_SUCCESS);
92 }
