/*************************************************************************\
*                  Copyright (C) Michael Kerrisk, 2014.                   *
*                                                                         *
* This program is free software. You may use, modify, and redistribute it *
* under the terms of the GNU Affero General Public License as published   *
* by the Free Software Foundation, either version 3 or (at your option)   *
* any later version. This program is distributed without any warranty.    *
* See the file COPYING.agpl-v3 for details.                               *
\*************************************************************************/

/* Listing 45-1 */

 1 #include <sys/types.h>
 2 #include <sys/ipc.h>
 3 #include <sys/msg.h>
 4 #include <sys/stat.h>
 5 #include "tlpi_hdr.h"
 6
 7 #define KEY_FILE "/some-path/some-file"
 8                                 /* Should be an existing file or one
 9                                    that this program creates */
10
11 int
12 main(int argc, char *argv[])
13 {
14     int msqid;
15     key_t key;
16     const int MQ_PERMS = S_IRUSR | S_IWUSR | S_IWGRP;   /* rw--w---- */
17
18     /* Optional code here to check if another server process is
19        already running */
20
21     /* Generate the key for the message queue */
22
23     key = ftok(KEY_FILE, 1);
24     if (key == -1)
25         errExit("ftok");
26
27     /* While msgget() fails, try creating the queue exclusively */
28
29     while ((msqid = msgget(key, IPC_CREAT | IPC_EXCL | MQ_PERMS)) ==
30             -1) {
31         if (errno == EEXIST) {          /* MQ with the same key already
32                                            exists - remove it and try again */
33             msqid = msgget(key, 0);
34             if (msqid == -1)
35                 errExit("msgget() failed to retrieve old queue ID");
36             if (msgctl(msqid, IPC_RMID, NULL) == -1)
37                 errExit("msgget() failed to delete old queue");
38             printf("Removed old message queue (id=%d)\n", msqid);
39
40         } else {                        /* Some other error --> give up */
41             errExit("msgget() failed");
42         }
43     }
44
45     /* Upon loop exit, we've successfully created the message queue,
46        and we can then carry on to do other work... */
47
48     exit(EXIT_SUCCESS);
49 }
