/*************************************************************************\
*                  Copyright (C) Michael Kerrisk, 2014.                   *
*                                                                         *
* This program is free software. You may use, modify, and redistribute it *
* under the terms of the GNU Affero General Public License as published   *
* by the Free Software Foundation, either version 3 or (at your option)   *
* any later version. This program is distributed without any warranty.    *
* See the file COPYING.agpl-v3 for details.                               *
\*************************************************************************/

/* Listing 46-1 */

 1 #include <sys/types.h>
 2 #include <sys/ipc.h>
 3 #include <sys/msg.h>
 4 #include <sys/stat.h>
 5 #include "tlpi_hdr.h"
 6
 7 static void             /* Print usage info, then exit */
 8 usageError(const char *progName, const char *msg)
 9 {
10     if (msg != NULL)
11         fprintf(stderr, "%s", msg);
12     fprintf(stderr, "Usage: %s [-cx] {-f pathname | -k key | -p} "
13                             "[octal-perms]\n", progName);
14     fprintf(stderr, "    -c           Use IPC_CREAT flag\n");
15     fprintf(stderr, "    -x           Use IPC_EXCL flag\n");
16     fprintf(stderr, "    -f pathname  Generate key using ftok()\n");
17     fprintf(stderr, "    -k key       Use 'key' as key\n");
18     fprintf(stderr, "    -p           Use IPC_PRIVATE key\n");
19     exit(EXIT_FAILURE);
20 }
21
22 int
23 main(int argc, char *argv[])
24 {
25     int numKeyFlags;            /* Counts -f, -k, and -p options */
26     int flags, msqid, opt;
27     unsigned int perms;
28     long lkey;
29     key_t key;
30
31     /* Parse command-line options and arguments */
32
33     numKeyFlags = 0;
34     flags = 0;
35
36     while ((opt = getopt(argc, argv, "cf:k:px")) != -1) {
37         switch (opt) {
38         case 'c':
39             flags |= IPC_CREAT;
40             break;
41
42         case 'f':               /* -f pathname */
43             key = ftok(optarg, 1);
44             if (key == -1)
45                 errExit("ftok");
46             numKeyFlags++;
47             break;
48
49         case 'k':               /* -k key (octal, decimal or hexadecimal) */
50             if (sscanf(optarg, "%li", &lkey) != 1)
51                 cmdLineErr("-k option requires a numeric argument\n");
52             key = lkey;
53             numKeyFlags++;
54             break;
55
56         case 'p':
57             key = IPC_PRIVATE;
58             numKeyFlags++;
59             break;
60
61         case 'x':
62             flags |= IPC_EXCL;
63             break;
64
65         default:
66             usageError(argv[0], "Bad option\n");
67         }
68     }
69
70     if (numKeyFlags != 1)
71         usageError(argv[0], "Exactly one of the options -f, -k, "
72                             "or -p must be supplied\n");
73
74     perms = (optind == argc) ? (S_IRUSR | S_IWUSR) :
75                 getInt(argv[optind], GN_BASE_8, "octal-perms");
76
77     msqid = msgget(key, flags | perms);
78     if (msqid == -1)
79         errExit("msgget");
80
81     printf("%d\n", msqid);
82     exit(EXIT_SUCCESS);
83 }
