/*************************************************************************\
*                  Copyright (C) Michael Kerrisk, 2014.                   *
*                                                                         *
* This program is free software. You may use, modify, and redistribute it *
* under the terms of the GNU Affero General Public License as published   *
* by the Free Software Foundation, either version 3 or (at your option)   *
* any later version. This program is distributed without any warranty.    *
* See the file COPYING.agpl-v3 for details.                               *
\*************************************************************************/

/* Listing 46-3 */

 1 #define _GNU_SOURCE             /* Get definition of MSG_EXCEPT */
 2 #include <sys/types.h>
 3 #include <sys/msg.h>
 4 #include "tlpi_hdr.h"
 5
 6 #define MAX_MTEXT 1024
 7
 8 struct mbuf {
 9     long mtype;                 /* Message type */
10     char mtext[MAX_MTEXT];      /* Message body */
11 };
12
13 static void
14 usageError(const char *progName, const char *msg)
15 {
16     if (msg != NULL)
17         fprintf(stderr, "%s", msg);
18     fprintf(stderr, "Usage: %s [options] msqid [max-bytes]\n", progName);
19     fprintf(stderr, "Permitted options are:\n");
20     fprintf(stderr, "    -e       Use MSG_NOERROR flag\n");
21     fprintf(stderr, "    -t type  Select message of given type\n");
22     fprintf(stderr, "    -n       Use IPC_NOWAIT flag\n");
23 #ifdef MSG_EXCEPT
24     fprintf(stderr, "    -x       Use MSG_EXCEPT flag\n");
25 #endif
26     exit(EXIT_FAILURE);
27 }
28
29 int
30 main(int argc, char *argv[])
31 {
32     int msqid, flags, type;
33     ssize_t msgLen;
34     size_t maxBytes;
35     struct mbuf msg;            /* Message buffer for msgrcv() */
36     int opt;                    /* Option character from getopt() */
37
38     /* Parse command-line options and arguments */
39
40     flags = 0;
41     type = 0;
42     while ((opt = getopt(argc, argv, "ent:x")) != -1) {
43         switch (opt) {
44         case 'e':       flags |= MSG_NOERROR;   break;
45         case 'n':       flags |= IPC_NOWAIT;    break;
46         case 't':       type = atoi(optarg);    break;
47 #ifdef MSG_EXCEPT
48         case 'x':       flags |= MSG_EXCEPT;    break;
49 #endif
50         default:        usageError(argv[0], NULL);
51         }
52     }
53
54     if (argc < optind + 1 || argc > optind + 2)
55         usageError(argv[0], "Wrong number of arguments\n");
56
57     msqid = getInt(argv[optind], 0, "msqid");
58     maxBytes = (argc > optind + 1) ?
59                 getInt(argv[optind + 1], 0, "max-bytes") : MAX_MTEXT;
60
61     /* Get message and display on stdout */
62
63     msgLen = msgrcv(msqid, &msg, maxBytes, type, flags);
64     if (msgLen == -1)
65         errExit("msgrcv");
66
67     printf("Received: type=%ld; length=%ld", msg.mtype, (long) msgLen);
68     if (msgLen > 0)
69         printf("; body=%s", msg.mtext);
70     printf("\n");
71
72     exit(EXIT_SUCCESS);
73 }
