/*************************************************************************\
*                  Copyright (C) Michael Kerrisk, 2014.                   *
*                                                                         *
* This program is free software. You may use, modify, and redistribute it *
* under the terms of the GNU Affero General Public License as published   *
* by the Free Software Foundation, either version 3 or (at your option)   *
* any later version. This program is distributed without any warranty.    *
* See the file COPYING.agpl-v3 for details.                               *
\*************************************************************************/

/* Listing 46-2 */

 1 #include <sys/types.h>
 2 #include <sys/msg.h>
 3 #include "tlpi_hdr.h"
 4
 5 #define MAX_MTEXT 1024
 6
 7 struct mbuf {
 8     long mtype;                         /* Message type */
 9     char mtext[MAX_MTEXT];              /* Message body */
10 };
11
12 static void             /* Print (optional) message, then usage description */
13 usageError(const char *progName, const char *msg)
14 {
15     if (msg != NULL)
16         fprintf(stderr, "%s", msg);
17     fprintf(stderr, "Usage: %s [-n] msqid msg-type [msg-text]\n", progName);
18     fprintf(stderr, "    -n       Use IPC_NOWAIT flag\n");
19     exit(EXIT_FAILURE);
20 }
21
22 int
23 main(int argc, char *argv[])
24 {
25     int msqid, flags, msgLen;
26     struct mbuf msg;                    /* Message buffer for msgsnd() */
27     int opt;                            /* Option character from getopt() */
28
29     /* Parse command-line options and arguments */
30
31     flags = 0;
32     while ((opt = getopt(argc, argv, "n")) != -1) {
33         if (opt == 'n')
34             flags |= IPC_NOWAIT;
35         else
36             usageError(argv[0], NULL);
37     }
38
39     if (argc < optind + 2 || argc > optind + 3)
40         usageError(argv[0], "Wrong number of arguments\n");
41
42     msqid = getInt(argv[optind], 0, "msqid");
43     msg.mtype = getInt(argv[optind + 1], 0, "msg-type");
44
45     if (argc > optind + 2) {            /* 'msg-text' was supplied */
46         msgLen = strlen(argv[optind + 2]) + 1;
47         if (msgLen > MAX_MTEXT)
48             cmdLineErr("msg-text too long (max: %d characters)\n", MAX_MTEXT);
49
50         memcpy(msg.mtext, argv[optind + 2], msgLen);
51
52     } else {                            /* No 'msg-text' ==> zero-length msg */
53         msgLen = 0;
54     }
55
56     /* Send message */
57
58     if (msgsnd(msqid, &msg, msgLen, flags) == -1)
59         errExit("msgsnd");
60
61     exit(EXIT_SUCCESS);
62 }
