/*************************************************************************\
*                  Copyright (C) Michael Kerrisk, 2014.                   *
*                                                                         *
* This program is free software. You may use, modify, and redistribute it *
* under the terms of the GNU Affero General Public License as published   *
* by the Free Software Foundation, either version 3 or (at your option)   *
* any later version. This program is distributed without any warranty.    *
* See the file COPYING.agpl-v3 for details.                               *
\*************************************************************************/

/* Listing 47-1 */

 1 #include <sys/types.h>
 2 #include <sys/sem.h>
 3 #include <sys/stat.h>
 4 #include "curr_time.h"                  /* Declaration of currTime() */
 5 #include "semun.h"                      /* Definition of semun union */
 6 #include "tlpi_hdr.h"
 7
 8 int
 9 main(int argc, char *argv[])
10 {
11     int semid;
12
13     if (argc < 2 || argc > 3 || strcmp(argv[1], "--help") == 0)
14         usageErr("%s init-value\n"
15                  "   or: %s semid operation\n", argv[0], argv[0]);
16
17     if (argc == 2) {            /* Create and initialize semaphore */
18         union semun arg;
19
20         semid = semget(IPC_PRIVATE, 1, S_IRUSR | S_IWUSR);
21         if (semid == -1)
22             errExit("semid");
23
24         arg.val = getInt(argv[1], 0, "init-value");
25         if (semctl(semid, /* semnum= */ 0, SETVAL, arg) == -1)
26             errExit("semctl");
27
28         printf("Semaphore ID = %d\n", semid);
29
30     } else {                    /* Perform an operation on first semaphore */
31
32         struct sembuf sop;              /* Structure defining operation */
33
34         semid = getInt(argv[1], 0, "semid");
35
36         sop.sem_num = 0;                /* Specifies first semaphore in set */
37         sop.sem_op = getInt(argv[2], 0, "operation");
38                                         /* Add, subtract, or wait for 0 */
39         sop.sem_flg = 0;                /* No special options for operation */
40
41         printf("%ld: about to semop at  %s\n", (long) getpid(), currTime("%T"));
42         if (semop(semid, &sop, 1) == -1)
43             errExit("semop");
44
45         printf("%ld: semop completed at %s\n", (long) getpid(), currTime("%T"));
46     }
47
48     exit(EXIT_SUCCESS);
49 }
