/*************************************************************************\
*                  Copyright (C) Michael Kerrisk, 2014.                   *
*                                                                         *
* This program is free software. You may use, modify, and redistribute it *
* under the terms of the GNU Affero General Public License as published   *
* by the Free Software Foundation, either version 3 or (at your option)   *
* any later version. This program is distributed without any warranty.    *
* See the file COPYING.agpl-v3 for details.                               *
\*************************************************************************/

/* Listing 47-6 */

 1 #include <sys/types.h>
 2 #include <sys/sem.h>
 3 #include <sys/stat.h>
 4 #include "semun.h"                      /* Definition of semun union */
 5 #include "tlpi_hdr.h"
 6
 7 int
 8 main(int argc, char *argv[])
 9 {
10     int semid, key, perms;
11     struct sembuf sops[2];
12
13     if (argc != 2 || strcmp(argv[1], "--help") == 0)
14         usageErr("%s sem-op\n", argv[0]);
15
16     key = 12345;
17     perms = S_IRUSR | S_IWUSR;
18
19     semid = semget(key, 1, IPC_CREAT | IPC_EXCL | perms);
20
21     if (semid != -1) {                  /* Successfully created the semaphore */
22         union semun arg;
23         struct sembuf sop;
24
25         arg.val = 0;                    /* So initialize it to 0 */
26         if (semctl(semid, 0, SETVAL, arg) == -1)
27             errExit("semctl 1");
28
29         /* Perform a "no-op" semaphore operation - changes sem_otime
30            so other processes can see we've initialized the set. */
31
32         sop.sem_num = 0;                /* Operate on semaphore 0 */
33         sop.sem_op = 0;                 /* Wait for value to equal 0 */
34         sop.sem_flg = 0;
35         if (semop(semid, &sop, 1) == -1)
36             errExit("semop");
37
38     } else {                            /* We didn't create the semaphore set */
39
40         if (errno != EEXIST) {          /* Unexpected error from semget() */
41             errExit("semget 1");
42
43         } else {                        /* Someone else already created it */
44             const int MAX_TRIES = 10;
45             int j;
46             union semun arg;
47             struct semid_ds ds;
48
49             semid = semget(key, 1, perms);      /* So just get ID */
50             if (semid == -1)
51                 errExit("semget 2");
52
53             /* Wait until another process has called semop() */
54
55             arg.buf = &ds;
56             for (j = 0; j < MAX_TRIES; j++) {
57                 if (semctl(semid, 0, IPC_STAT, arg) == -1)
58                     errExit("semctl 2");
59
60                 if (ds.sem_otime != 0)          /* Semop() performed? */
61                     break;                      /* Yes, quit loop */
62                 sleep(1);                       /* If not, wait and retry */
63             }
64
65             if (ds.sem_otime == 0)              /* Loop ran to completion! */
66                 fatal("Existing semaphore not initialized");
67         }
68     }
69
70     /* Now perform some operation on the semaphore */
71
72     sops[0].sem_num = 0;                        /* Operate on semaphore 0... */
73     sops[0].sem_op = getInt(argv[1], 0, "sem-op");
74     sops[0].sem_flg = 0;
75     if (semop(semid, sops, 1) == -1)
76         errExit("semop");
77
78     exit(EXIT_SUCCESS);
79 }
