/*************************************************************************\
*                  Copyright (C) Michael Kerrisk, 2014.                   *
*                                                                         *
* This program is free software. You may use, modify, and redistribute it *
* under the terms of the GNU Affero General Public License as published   *
* by the Free Software Foundation, either version 3 or (at your option)   *
* any later version. This program is distributed without any warranty.    *
* See the file COPYING.agpl-v3 for details.                               *
\*************************************************************************/

/* Listing 48-2 */

 1 #include "semun.h"              /* Definition of semun union */
 2 #include "svshm_xfr.h"
 3
 4 int
 5 main(int argc, char *argv[])
 6 {
 7     int semid, shmid, bytes, xfrs;
 8     struct shmseg *shmp;
 9     union semun dummy;
10
11     semid = semget(SEM_KEY, 2, IPC_CREAT | OBJ_PERMS);
12     if (semid == -1)
13         errExit("semget");
14
15     if (initSemAvailable(semid, WRITE_SEM) == -1)
16         errExit("initSemAvailable");
17     if (initSemInUse(semid, READ_SEM) == -1)
18         errExit("initSemInUse");
19
20     shmid = shmget(SHM_KEY, sizeof(struct shmseg), IPC_CREAT | OBJ_PERMS);
21     if (shmid == -1)
22         errExit("shmget");
23
24     shmp = shmat(shmid, NULL, 0);
25     if (shmp == (void *) -1)
26         errExit("shmat");
27
28     /* Transfer blocks of data from stdin to shared memory */
29
30     for (xfrs = 0, bytes = 0; ; xfrs++, bytes += shmp->cnt) {
31         if (reserveSem(semid, WRITE_SEM) == -1)         /* Wait for our turn */
32             errExit("reserveSem");
33
34         shmp->cnt = read(STDIN_FILENO, shmp->buf, BUF_SIZE);
35         if (shmp->cnt == -1)
36             errExit("read");
37
38         if (releaseSem(semid, READ_SEM) == -1)          /* Give reader a turn */
39             errExit("releaseSem");
40
41         /* Have we reached EOF? We test this after giving the reader
42            a turn so that it can see the 0 value in shmp->cnt. */
43
44         if (shmp->cnt == 0)
45             break;
46     }
47
48     /* Wait until reader has let us have one more turn. We then know
49        reader has finished, and so we can delete the IPC objects. */
50
51     if (reserveSem(semid, WRITE_SEM) == -1)
52         errExit("reserveSem");
53
54     if (semctl(semid, 0, IPC_RMID, dummy) == -1)
55         errExit("semctl");
56     if (shmdt(shmp) == -1)
57         errExit("shmdt");
58     if (shmctl(shmid, IPC_RMID, 0) == -1)
59         errExit("shmctl");
60
61     fprintf(stderr, "Sent %d bytes (%d xfrs)\n", bytes, xfrs);
62     exit(EXIT_SUCCESS);
63 }
