/*************************************************************************\
*                  Copyright (C) Michael Kerrisk, 2014.                   *
*                                                                         *
* This program is free software. You may use, modify, and redistribute it *
* under the terms of the GNU Affero General Public License as published   *
* by the Free Software Foundation, either version 3 or (at your option)   *
* any later version. This program is distributed without any warranty.    *
* See the file COPYING.agpl-v3 for details.                               *
\*************************************************************************/

/* Solution for Exercise 12-1 */

 1 #define _GNU_SOURCE
 2 #include <limits.h>
 3 #include <sys/stat.h>
 4 #include <dirent.h>
 5 #include <ctype.h>
 6 #include "ugid_functions.h"
 7 #include "tlpi_hdr.h"
 8
 9 #define MAX_LINE 1000
10
11 int
12 main(int argc, char *argv[])
13 {
14     DIR *dirp;
15     struct dirent *dp;
16     char path[PATH_MAX];
17     char line[MAX_LINE], cmd[MAX_LINE];
18     FILE *fp;
19     char *p;
20     uid_t uid, checkedUid;
21     Boolean gotName, gotUid;
22
23     if (argc < 2 || strcmp(argv[1], "--help") == 0)
24         usageErr("%s username\n", argv[0]);
25
26     checkedUid = userIdFromName(argv[1]);
27     if (checkedUid == -1)
28         cmdLineErr("Bad username: %s\n", argv[1]);
29
30     dirp = opendir("/proc");
31     if (dirp == NULL)
32         errExit("opendir");
33
34     for (;;) {
35         errno = 0;              /* To distinguish error from end-of-directory */
36         dp = readdir(dirp);
37         if (dp == NULL) {
38             if (errno != 0)
39                 errExit("readdir");
40             else
41                 break;
42         }
43
44         if (dp->d_type != DT_DIR || !isdigit((unsigned char) dp->d_name[0]))
45             continue;
46
47         snprintf(path, PATH_MAX, "/proc/%s/status", dp->d_name);
48
49         fp = fopen(path, "r");
50         if (fp == NULL)
51             continue;           /* Ignore errors: fopen() might fail if
52                                    process has just terminated */
53
54         gotName = FALSE;
55         gotUid = FALSE;
56         while (!gotName || !gotUid) {
57             if (fgets(line, MAX_LINE, fp) == NULL)
58                 break;
59
60             /* The "Name:" line contains the name of the command that
61                this process is running */
62
63             if (strncmp(line, "Name:", 5) == 0) {
64                 for (p = line + 5; *p != '\0' && isspace((unsigned char) *p); )
65                     p++;
66                 strncpy(cmd, p, MAX_LINE - 1);
67                 cmd[MAX_LINE -1] = '\0';        /* Ensure null-terminated */
68
69                 gotName = TRUE;
70             }
71
72             /* The "Uid:" line contains the real, effective, saved set-,
73                and file-system user IDs */
74
75             if (strncmp(line, "Uid:", 4) == 0) {
76                 uid = strtol(line + 4, NULL, 10);
77                 gotUid = TRUE;
78             }
79         }
80
81         fclose(fp);
82
83         /* If we found a username and a UID, and the UID matches,
84            then display the PID and command name */
85
86         if (gotName && gotUid && uid == checkedUid)
87             printf("%5s %s", dp->d_name, cmd);
88     }
89
90     exit(EXIT_SUCCESS);
91 }
