/*************************************************************************\
*                  Copyright (C) Michael Kerrisk, 2014.                   *
*                                                                         *
* This program is free software. You may use, modify, and redistribute it *
* under the terms of the GNU Affero General Public License as published   *
* by the Free Software Foundation, either version 3 or (at your option)   *
* any later version. This program is distributed without any warranty.    *
* See the file COPYING.agpl-v3 for details.                               *
\*************************************************************************/

/* Solution for Exercise 31-1 */

 1 #include <pthread.h>
 2 #include "tlpi_hdr.h"
 3
 4 struct once_struct {            /* Our equivalent of pthread_once_t */
 5     pthread_mutex_t mtx;
 6     int called;
 7 };
 8
 9 #define ONCE_INITIALISER { PTHREAD_MUTEX_INITIALIZER, 0 }
10
11 struct once_struct once = ONCE_INITIALISER;
12
13 static int
14 one_time_init(struct once_struct *once_control, void (*init)(void))
15 {
16     int s;
17
18     s = pthread_mutex_lock(&(once_control->mtx));
19     if (s == -1)
20         errExitEN(s, "pthread_mutex_lock");
21
22     if (!once_control->called) {
23         (*init)();
24         once_control->called = 1;
25     }
26
27     s = pthread_mutex_unlock(&(once_control->mtx));
28     if (s == -1)
29         errExitEN(s, "pthread_mutex_unlock");
30
31     return 0;
32 }
33
34 /* Remaining code is for testing one_time_init() */
35
36 static void
37 init_func()
38 {
39     /* We should see this message only once, no matter how many
40        times one_time_init() is called */
41
42     printf("Called init_func()\n");
43 }
44
45 static void *
46 threadFunc(void *arg)
47 {
48     /* The following allows us to verify that even if a single thread calls
49        one_time_init() multiple times, init_func() is called only once */
50
51     one_time_init(&once, init_func);
52     one_time_init(&once, init_func);
53     return NULL;
54 }
55
56 int
57 main(int argc, char *argv[])
58 {
59     pthread_t t1, t2;
60     int s;
61
62     /* Create two threads, both of which will call one_time_init() */
63
64     s = pthread_create(&t1, NULL, threadFunc, (void *) 1);
65     if (s != 0)
66         errExitEN(s, "pthread_create");
67
68     s = pthread_create(&t2, NULL, threadFunc, (void *) 2);
69     if (s != 0)
70         errExitEN(s, "pthread_create");
71
72     s = pthread_join(t1, NULL);
73     if (s != 0)
74         errExitEN(s, "pthread_join");
75     printf("First thread returned\n");
76
77     s = pthread_join(t2, NULL);
78     if (s != 0)
79         errExitEN(s, "pthread_join");
80     printf("Second thread returned\n");
81
82     exit(EXIT_SUCCESS);
83 }
