/*************************************************************************\
*                  Copyright (C) Michael Kerrisk, 2014.                   *
*                                                                         *
* This program is free software. You may use, modify, and redistribute it *
* under the terms of the GNU Affero General Public License as published   *
* by the Free Software Foundation, either version 3 or (at your option)   *
* any later version. This program is distributed without any warranty.    *
* See the file COPYING.agpl-v3 for details.                               *
\*************************************************************************/

/* Listing 31-3 */

 1 #define _GNU_SOURCE                 /* Get '_sys_nerr' and '_sys_errlist'
 2                                        declarations from <stdio.h> */
 3 #include <stdio.h>
 4 #include <string.h>                 /* Get declaration of strerror() */
 5 #include <pthread.h>
 6 #include "tlpi_hdr.h"
 7
 8 static pthread_once_t once = PTHREAD_ONCE_INIT;
 9 static pthread_key_t strerrorKey;
10
11 #define MAX_ERROR_LEN 256           /* Maximum length of string in per-thread
12                                        buffer returned by strerror() */
13
14 static void                         /* Free thread-specific data buffer */
15 destructor(void *buf)
16 {
17     free(buf);
18 }
19
20 static void                         /* One-time key creation function */
21 createKey(void)
22 {
23     int s;
24
25     /* Allocate a unique thread-specific data key and save the address
26        of the destructor for thread-specific data buffers */
27
28     s = pthread_key_create(&strerrorKey, destructor);
29     if (s != 0)
30         errExitEN(s, "pthread_key_create");
31 }
32
33 char *
34 strerror(int err)
35 {
36     int s;
37     char *buf;
38
39     /* Make first caller allocate key for thread-specific data */
40
41     s = pthread_once(&once, createKey);
42     if (s != 0)
43         errExitEN(s, "pthread_once");
44
45     buf = pthread_getspecific(strerrorKey);
46     if (buf == NULL) {          /* If first call from this thread, allocate
47                                    buffer for thread, and save its location */
48         buf = malloc(MAX_ERROR_LEN);
49         if (buf == NULL)
50             errExit("malloc");
51
52         s = pthread_setspecific(strerrorKey, buf);
53         if (s != 0)
54             errExitEN(s, "pthread_setspecific");
55     }
56
57     if (err < 0 || err >= _sys_nerr || _sys_errlist[err] == NULL) {
58         snprintf(buf, MAX_ERROR_LEN, "Unknown error %d", err);
59     } else {
60         strncpy(buf, _sys_errlist[err], MAX_ERROR_LEN - 1);
61         buf[MAX_ERROR_LEN - 1] = '\0';          /* Ensure null termination */
62     }
63
64     return buf;
65 }
