/*************************************************************************\
*                  Copyright (C) Michael Kerrisk, 2014.                   *
*                                                                         *
* This program is free software. You may use, modify, and redistribute it *
* under the terms of the GNU Affero General Public License as published   *
* by the Free Software Foundation, either version 3 or (at your option)   *
* any later version. This program is distributed without any warranty.    *
* See the file COPYING.agpl-v3 for details.                               *
\*************************************************************************/

/* Listing 30-4 */

  1 #include <pthread.h>
  2 #include "tlpi_hdr.h"
  3
  4 static pthread_cond_t threadDied = PTHREAD_COND_INITIALIZER;
  5 static pthread_mutex_t threadMutex = PTHREAD_MUTEX_INITIALIZER;
  6                 /* Protects all of the following global variables */
  7
  8 static int totThreads = 0;      /* Total number of threads created */
  9 static int numLive = 0;         /* Total number of threads still alive or
 10                                    terminated but not yet joined */
 11 static int numUnjoined = 0;     /* Number of terminated threads that
 12                                    have not yet been joined */
 13 enum tstate {                   /* Thread states */
 14     TS_ALIVE,                   /* Thread is alive */
 15     TS_TERMINATED,              /* Thread terminated, not yet joined */
 16     TS_JOINED                   /* Thread terminated, and joined */
 17 };
 18
 19 static struct {                 /* Info about each thread */
 20     pthread_t tid;              /* ID of this thread */
 21     enum tstate state;          /* Thread state (TS_* constants above) */
 22     int sleepTime;              /* Number seconds to live before terminating */
 23 } *thread;
 24
 25 static void *                   /* Start function for thread */
 26 threadFunc(void *arg)
 27 {
 28     int idx = (int) arg;
 29     int s;
 30
 31     sleep(thread[idx].sleepTime);       /* Simulate doing some work */
 32     printf("Thread %d terminating\n", idx);
 33
 34     s = pthread_mutex_lock(&threadMutex);
 35     if (s != 0)
 36         errExitEN(s, "pthread_mutex_lock");
 37
 38     numUnjoined++;
 39     thread[idx].state = TS_TERMINATED;
 40
 41     s = pthread_mutex_unlock(&threadMutex);
 42     if (s != 0)
 43         errExitEN(s, "pthread_mutex_unlock");
 44     s = pthread_cond_signal(&threadDied);
 45     if (s != 0)
 46         errExitEN(s, "pthread_cond_signal");
 47
 48     return NULL;
 49 }
 50
 51 int
 52 main(int argc, char *argv[])
 53 {
 54     int s, idx;
 55
 56     if (argc < 2 || strcmp(argv[1], "--help") == 0)
 57         usageErr("%s nsecs...\n", argv[0]);
 58
 59     thread = calloc(argc - 1, sizeof(*thread));
 60     if (thread == NULL)
 61         errExit("calloc");
 62
 63     /* Create all threads */
 64
 65     for (idx = 0; idx < argc - 1; idx++) {
 66         thread[idx].sleepTime = getInt(argv[idx + 1], GN_NONNEG, NULL);
 67         thread[idx].state = TS_ALIVE;
 68         s = pthread_create(&thread[idx].tid, NULL, threadFunc, (void *) idx);
 69         if (s != 0)
 70             errExitEN(s, "pthread_create");
 71     }
 72
 73     totThreads = argc - 1;
 74     numLive = totThreads;
 75
 76     /* Join with terminated threads */
 77
 78     while (numLive > 0) {
 79         s = pthread_mutex_lock(&threadMutex);
 80         if (s != 0)
 81             errExitEN(s, "pthread_mutex_lock");
 82
 83         while (numUnjoined == 0) {
 84             s = pthread_cond_wait(&threadDied, &threadMutex);
 85             if (s != 0)
 86                 errExitEN(s, "pthread_cond_wait");
 87         }
 88
 89         for (idx = 0; idx < totThreads; idx++) {
 90             if (thread[idx].state == TS_TERMINATED){
 91                 s = pthread_join(thread[idx].tid, NULL);
 92                 if (s != 0)
 93                     errExitEN(s, "pthread_join");
 94
 95                 thread[idx].state = TS_JOINED;
 96                 numLive--;
 97                 numUnjoined--;
 98
 99                 printf("Reaped thread %d (numLive=%d)\n", idx, numLive);
100             }
101         }
102
103         s = pthread_mutex_unlock(&threadMutex);
104         if (s != 0)
105             errExitEN(s, "pthread_mutex_unlock");
106     }
107
108     exit(EXIT_SUCCESS);
109 }
