/*************************************************************************\
*                  Copyright (C) Michael Kerrisk, 2014.                   *
*                                                                         *
* This program is free software. You may use, modify, and redistribute it *
* under the terms of the GNU Affero General Public License as published   *
* by the Free Software Foundation, either version 3 or (at your option)   *
* any later version. This program is distributed without any warranty.    *
* See the file COPYING.agpl-v3 for details.                               *
\*************************************************************************/

/* Listing 23-8 */

 1 #include <sys/timerfd.h>
 2 #include <time.h>
 3 #include <stdint.h>                     /* Definition of uint64_t */
 4 #include "itimerspec_from_str.h"        /* Declares itimerspecFromStr() */
 5 #include "tlpi_hdr.h"
 6
 7 int
 8 main(int argc, char *argv[])
 9 {
10     struct itimerspec ts;
11     struct timespec start, now;
12     int maxExp, fd, secs, nanosecs;
13     uint64_t numExp, totalExp;
14     ssize_t s;
15
16     if (argc < 2 || strcmp(argv[1], "--help") == 0)
17         usageErr("%s secs[/nsecs][:int-secs[/int-nsecs]] [max-exp]\n", argv[0]);
18
19     itimerspecFromStr(argv[1], &ts);
20     maxExp = (argc > 2) ? getInt(argv[2], GN_GT_0, "max-exp") : 1;
21
22     fd = timerfd_create(CLOCK_REALTIME, 0);
23     if (fd == -1)
24         errExit("timerfd_create");
25
26     if (timerfd_settime(fd, 0, &ts, NULL) == -1)
27         errExit("timerfd_settime");
28
29     if (clock_gettime(CLOCK_MONOTONIC, &start) == -1)
30         errExit("clock_gettime");
31
32     for (totalExp = 0; totalExp < maxExp;) {
33
34         /* Read number of expirations on the timer, and then display
35            time elapsed since timer was started, followed by number
36            of expirations read and total expirations so far. */
37
38         s = read(fd, &numExp, sizeof(uint64_t));
39         if (s != sizeof(uint64_t))
40             errExit("read");
41
42         totalExp += numExp;
43
44         if (clock_gettime(CLOCK_MONOTONIC, &now) == -1)
45             errExit("clock_gettime");
46
47         secs = now.tv_sec - start.tv_sec;
48         nanosecs = now.tv_nsec - start.tv_nsec;
49         if (nanosecs < 0) {
50             secs--;
51             nanosecs += 1000000000;
52         }
53
54         printf("%d.%03d: expirations read: %llu; total=%llu\n",
55                 secs, (nanosecs + 500000) / 1000000,
56                 (unsigned long long) numExp, (unsigned long long) totalExp);
57     }
58
59     exit(EXIT_SUCCESS);
60 }
