/*************************************************************************\
*                  Copyright (C) Michael Kerrisk, 2014.                   *
*                                                                         *
* This program is free software. You may use, modify, and redistribute it *
* under the terms of the GNU Affero General Public License as published   *
* by the Free Software Foundation, either version 3 or (at your option)   *
* any later version. This program is distributed without any warranty.    *
* See the file COPYING.agpl-v3 for details.                               *
\*************************************************************************/

/* Listing 23-7 */

 1 #include <signal.h>
 2 #include <time.h>
 3 #include <pthread.h>
 4 #include "curr_time.h"              /* Declares currTime() */
 5 #include "tlpi_hdr.h"
 6 #include "itimerspec_from_str.h"    /* Declares itimerspecFromStr() */
 7
 8 static pthread_mutex_t mtx = PTHREAD_MUTEX_INITIALIZER;
 9 static pthread_cond_t cond = PTHREAD_COND_INITIALIZER;
10
11 static int expireCnt = 0;           /* Number of expirations of all timers */
12
13 static void                         /* Thread notification function */
14 threadFunc(union sigval sv)
15 {
16     timer_t *tidptr;
17     int s;
18
19     tidptr = sv.sival_ptr;
20
21     printf("[%s] Thread notify\n", currTime("%T"));
22     printf("    timer ID=%ld\n", (long) *tidptr);
23     printf("    timer_getoverrun()=%d\n", timer_getoverrun(*tidptr));
24
25     /* Increment counter variable shared with main thread and signal
26        condition variable to notify main thread of the change. */
27
28     s = pthread_mutex_lock(&mtx);
29     if (s != 0)
30         errExitEN(s, "pthread_mutex_lock");
31
32     expireCnt += 1 + timer_getoverrun(*tidptr);
33
34     s = pthread_mutex_unlock(&mtx);
35     if (s != 0)
36         errExitEN(s, "pthread_mutex_unlock");
37
38     s = pthread_cond_signal(&cond);
39     if (s != 0)
40         errExitEN(s, "pthread_cond_signal");
41 }
42
43 int
44 main(int argc, char *argv[])
45 {
46     struct sigevent sev;
47     struct itimerspec ts;
48     timer_t *tidlist;
49     int s, j;
50
51     if (argc < 2)
52         usageErr("%s secs[/nsecs][:int-secs[/int-nsecs]]...\n", argv[0]);
53
54     tidlist = calloc(argc - 1, sizeof(timer_t));
55     if (tidlist == NULL)
56         errExit("malloc");
57
58     sev.sigev_notify = SIGEV_THREAD;            /* Notify via thread */
59     sev.sigev_notify_function = threadFunc;     /* Thread start function */
60     sev.sigev_notify_attributes = NULL;
61             /* Could be pointer to pthread_attr_t structure */
62
63     /* Create and start one timer for each command-line argument */
64
65     for (j = 0; j < argc - 1; j++) {
66         itimerspecFromStr(argv[j + 1], &ts);
67
68         sev.sigev_value.sival_ptr = &tidlist[j];
69                 /* Passed as argument to threadFunc() */
70
71         if (timer_create(CLOCK_REALTIME, &sev, &tidlist[j]) == -1)
72             errExit("timer_create");
73         printf("Timer ID: %ld (%s)\n", (long) tidlist[j], argv[j + 1]);
74
75         if (timer_settime(tidlist[j], 0, &ts, NULL) == -1)
76             errExit("timer_settime");
77     }
78
79     /* The main thread waits on a condition variable that is signaled
80        on each invocation of the thread notification function. We
81        print a message so that the user can see that this occurred. */
82
83     s = pthread_mutex_lock(&mtx);
84     if (s != 0)
85         errExitEN(s, "pthread_mutex_lock");
86
87     for (;;) {
88         s = pthread_cond_wait(&cond, &mtx);
89         if (s != 0)
90             errExitEN(s, "pthread_cond_wait");
91         printf("main(): expireCnt = %d\n", expireCnt);
92     }
93 }
