/*************************************************************************\
*                  Copyright (C) Michael Kerrisk, 2014.                   *
*                                                                         *
* This program is free software. You may use, modify, and redistribute it *
* under the terms of the GNU Affero General Public License as published   *
* by the Free Software Foundation, either version 3 or (at your option)   *
* any later version. This program is distributed without any warranty.    *
* See the file COPYING.agpl-v3 for details.                               *
\*************************************************************************/

/* Listing 23-3 */

 1 #define _POSIX_C_SOURCE 199309
 2 #include <sys/time.h>
 3 #include <time.h>
 4 #include <signal.h>
 5 #include "tlpi_hdr.h"
 6
 7 static void
 8 sigintHandler(int sig)
 9 {
10     return;                     /* Just interrupt nanosleep() */
11 }
12
13 int
14 main(int argc, char *argv[])
15 {
16     struct timeval start, finish;
17     struct timespec request, remain;
18     struct sigaction sa;
19     int s;
20
21     if (argc != 3 || strcmp(argv[1], "--help") == 0)
22         usageErr("%s secs nanosecs\n", argv[0]);
23
24     request.tv_sec = getLong(argv[1], 0, "secs");
25     request.tv_nsec = getLong(argv[2], 0, "nanosecs");
26
27     /* Allow SIGINT handler to interrupt nanosleep() */
28
29     sigemptyset(&sa.sa_mask);
30     sa.sa_flags = 0;
31     sa.sa_handler = sigintHandler;
32     if (sigaction(SIGINT, &sa, NULL) == -1)
33         errExit("sigaction");
34
35     if (gettimeofday(&start, NULL) == -1)
36         errExit("gettimeofday");
37
38     for (;;) {
39         s = nanosleep(&request, &remain);
40         if (s == -1 && errno != EINTR)
41             errExit("nanosleep");
42
43         if (gettimeofday(&finish, NULL) == -1)
44             errExit("gettimeofday");
45         printf("Slept for: %9.6f secs\n", finish.tv_sec - start.tv_sec +
46                         (finish.tv_usec - start.tv_usec) / 1000000.0);
47
48         if (s == 0)
49             break;                      /* nanosleep() completed */
50
51         printf("Remaining: %2ld.%09ld\n", (long) remain.tv_sec, remain.tv_nsec);
52         request = remain;               /* Next sleep is with remaining time */
53     }
54
55     printf("Sleep complete\n");
56     exit(EXIT_SUCCESS);
57 }
