/*************************************************************************\
*                  Copyright (C) Michael Kerrisk, 2014.                   *
*                                                                         *
* This program is free software. You may use, modify, and redistribute it *
* under the terms of the GNU Affero General Public License as published   *
* by the Free Software Foundation, either version 3 or (at your option)   *
* any later version. This program is distributed without any warranty.    *
* See the file COPYING.agpl-v3 for details.                               *
\*************************************************************************/

/* Listing 8-2 */

 1 #define _BSD_SOURCE     /* Get getpass() declaration from <unistd.h> */
 2 #define _XOPEN_SOURCE   /* Get crypt() declaration from <unistd.h> */
 3 #include <unistd.h>
 4 #include <limits.h>
 5 #include <pwd.h>
 6 #include <shadow.h>
 7 #include "tlpi_hdr.h"
 8
 9 int
10 main(int argc, char *argv[])
11 {
12     char *username, *password, *encrypted, *p;
13     struct passwd *pwd;
14     struct spwd *spwd;
15     Boolean authOk;
16     size_t len;
17     long lnmax;
18
19     lnmax = sysconf(_SC_LOGIN_NAME_MAX);
20     if (lnmax == -1)                    /* If limit is indeterminate */
21         lnmax = 256;                    /* make a guess */
22
23     username = malloc(lnmax);
24     if (username == NULL)
25         errExit("malloc");
26
27     printf("Username: ");
28     fflush(stdout);
29     if (fgets(username, lnmax, stdin) == NULL)
30         exit(EXIT_FAILURE);             /* Exit on EOF */
31
32     len = strlen(username);
33     if (username[len - 1] == '\n')
34         username[len - 1] = '\0';       /* Remove trailing '\n' */
35
36     pwd = getpwnam(username);
37     if (pwd == NULL)
38         fatal("couldn't get password record");
39     spwd = getspnam(username);
40     if (spwd == NULL && errno == EACCES)
41         fatal("no permission to read shadow password file");
42
43     if (spwd != NULL)           /* If there is a shadow password record */
44         pwd->pw_passwd = spwd->sp_pwdp;     /* Use the shadow password */
45
46     password = getpass("Password: ");
47
48     /* Encrypt password and erase cleartext version immediately */
49
50     encrypted = crypt(password, pwd->pw_passwd);
51     for (p = password; *p != '\0'; )
52         *p++ = '\0';
53
54     if (encrypted == NULL)
55         errExit("crypt");
56
57     authOk = strcmp(encrypted, pwd->pw_passwd) == 0;
58     if (!authOk) {
59         printf("Incorrect password\n");
60         exit(EXIT_FAILURE);
61     }
62
63     printf("Successfully authenticated: UID=%ld\n", (long) pwd->pw_uid);
64
65     /* Now do authenticated work... */
66
67     exit(EXIT_SUCCESS);
68 }
