/*************************************************************************\
*                  Copyright (C) Michael Kerrisk, 2014.                   *
*                                                                         *
* This program is free software. You may use, modify, and redistribute it *
* under the terms of the GNU Affero General Public License as published   *
* by the Free Software Foundation, either version 3 or (at your option)   *
* any later version. This program is distributed without any warranty.    *
* See the file COPYING.agpl-v3 for details.                               *
\*************************************************************************/

/* Listing 50-2 */

 1 #define _BSD_SOURCE             /* Get mincore() declaration and MAP_ANONYMOUS
 2                                    definition from <sys/mman.h> */
 3 #include <sys/mman.h>
 4 #include "tlpi_hdr.h"
 5
 6 /* Display residency of pages in range [addr .. (addr + length - 1)] */
 7
 8 static void
 9 displayMincore(char *addr, size_t length)
10 {
11     unsigned char *vec;
12     long pageSize, numPages, j;
13
14     pageSize = sysconf(_SC_PAGESIZE);
15
16     numPages = (length + pageSize - 1) / pageSize;
17     vec = malloc(numPages);
18     if (vec == NULL)
19         errExit("malloc");
20
21     if (mincore(addr, length, vec) == -1)
22         errExit("mincore");
23
24     for (j = 0; j < numPages; j++) {
25         if (j % 64 == 0)
26             printf("%s%10p: ", (j == 0) ? "" : "\n", addr + (j * pageSize));
27         printf("%c", (vec[j] & 1) ? '*' : '.');
28     }
29     printf("\n");
30
31     free(vec);
32 }
33
34 int
35 main(int argc, char *argv[])
36 {
37     char *addr;
38     size_t len, lockLen;
39     long pageSize, stepSize, j;
40
41     if (argc != 4 || strcmp(argv[1], "--help") == 0)
42         usageErr("%s num-pages lock-page-step lock-page-len\n", argv[0]);
43
44     pageSize = sysconf(_SC_PAGESIZE);
45     if (pageSize == -1)
46         errExit("sysconf(_SC_PAGESIZE)");
47
48     len =      getInt(argv[1], GN_GT_0, "num-pages") * pageSize;
49     stepSize = getInt(argv[2], GN_GT_0, "lock-page-step") * pageSize;
50     lockLen =  getInt(argv[3], GN_GT_0, "lock-page-len") * pageSize;
51
52     addr = mmap(NULL, len, PROT_READ, MAP_SHARED | MAP_ANONYMOUS, -1, 0);
53     if (addr == MAP_FAILED)
54         errExit("mmap");
55
56     printf("Allocated %ld (%#lx) bytes starting at %p\n",
57             (long) len, (unsigned long) len, addr);
58
59     printf("Before mlock:\n");
60     displayMincore(addr, len);
61
62     /* Lock pages specified by command-line arguments into memory */
63
64     for (j = 0; j + lockLen <= len; j += stepSize)
65         if (mlock(addr + j, lockLen) == -1)
66             errExit("mlock");
67
68     printf("After mlock:\n");
69     displayMincore(addr, len);
70
71     exit(EXIT_SUCCESS);
72 }
